


from reahl.web.fw import Region, Bookmark
from reahl.web.ui import TwoColumnPage, P, H, Panel, HMenu
from reahl.component.modelinterface import exposed, IntegerField


class WidgetRefreshApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')
        find = self.define_view(u'/', title=u'Refreshing widget')
        find.set_slot(u'main', HomePanel.factory())


class HomePanel(Panel):
    def __init__(self, view):
        super(HomePanel, self).__init__(view)

        panel = RefreshedPanel(view, u'my_refreshedpanel')
        bookmarks = [panel.get_bookmark(1),
                     panel.get_bookmark(2),
                     panel.get_bookmark(3)]

        self.add_child(H(view, 1, text=u'Refreshing widget'))
        self.add_child(HMenu.from_bookmarks(view, bookmarks))
        self.add_child(panel)


class RefreshedPanel(Panel):
    def __init__(self, view, css_id):
        super(RefreshedPanel, self).__init__(view, css_id=css_id)
        self.add_child(P(view, text=u'You selected link number %s' % self.selected))
        self.enable_refresh()

    @exposed
    def query_fields(self, fields):
        fields.selected = IntegerField(required=False, default=1)
        
    def get_bookmark(self, for_selected):
        return Bookmark.for_widget(u'Select %s' % for_selected, query_arguments={u'selected': for_selected})











from datetime import datetime

import elixir


from reahl.sqlalchemysupport import Session, metadata

from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, P, H, InputGroup
from reahl.component.modelinterface import exposed, EmailField, Field, Event, Action
from reahl.component.migration import Migration


class AddressBookApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')
        find = self.define_view(u'/', title=u'Addresses')
        find.set_slot(u'main', AddressBookPanel.factory())


class AddressBookPanel(Panel):
    def __init__(self, view):
        super(AddressBookPanel, self).__init__(view)

        self.add_child(H(view, 1, text=u'Addresses'))
        
        for address in Address.query.all():
            self.add_child(AddressBox(view, address))

        self.add_child(AddAddressForm(view))


class AddAddressForm(Form):
    def __init__(self, view):
        super(AddAddressForm, self).__init__(view, u'add_form')

        new_address = Address()

        grouped_inputs = self.add_child(InputGroup(view, label_text=u'Add an address'))
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_address.fields.name)) )
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_address.fields.email_address)) )

        self.define_event_handler(new_address.events.save)
        grouped_inputs.add_child(Button(self, new_address.events.save))


class AddressBox(Panel):
    def __init__(self, view, address):
        super(AddressBox, self).__init__(view)
        new = u' (new)' if address.added_today else u''
        self.add_child(P(view, text=u'%s: %s%s' % (address.name, address.email_address, new)))


class Address(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    elixir.using_mapper_options(save_on_init=False)
    
    email_address = elixir.Field(elixir.UnicodeText)
    name          = elixir.Field(elixir.UnicodeText)
    added_today   = elixir.Field(elixir.Boolean, required=True, default=True)

    @classmethod
    def clear_added_flags(cls):
        for address in cls.query.filter_by(added_today=True):
            address.added_today = False

    @exposed
    def fields(self, fields):
        fields.name = Field(label=u'Name', required=True)
        fields.email_address = EmailField(label=u'Email', required=True)

    @exposed
    def events(self, events):
        events.save = Event(label=u'Save', action=Action(self.save))

    def save(self):
        Session.add(self)




# Copyright 2013 Reahl Software Services (Pty) Ltd. All rights reserved.
#
#    This file is part of Reahl.
#
#    Reahl is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation; version 3 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import shutil
import pkg_resources
import codecs
import re

from reahl.dev.devshell import WorkspaceCommand


class Example(object):
    @classmethod
    def get_all(cls):
        parent = pkg_resources.resource_filename(u'reahl.doc', u'examples')
        examples = []
        for dirpath, dirnames, filenames in os.walk(parent): 
            current_package = dirpath[len(parent)+1:].replace(os.sep, u'.')
            if u'.reahlproject' in filenames:
                examples.append(Example(current_package))
                dirnames[:] = []
            else:
                for module in [os.path.splitext(f)[0] for f in filenames
                               if f.endswith(u'.py') and f != '__init__.py']:
                    if current_package:
                        module_path = u'%s.%s' % (current_package, module)
                    else:
                        module_path = module
                    examples.append(Example(module_path))
        return examples

    def __init__(self, name):
        self.name = name

    def is_package(self):
        name_as_path = self.name.replace(u'.', os.sep)
        return self.abs_path_to_package.endswith(name_as_path)

    @property
    def abs_path_to_package(self):
        return pkg_resources.resource_filename(u'.'.join([u'reahl.doc.examples', self.name]), u'')

    @property
    def module_name(self):
        return self.name.split(u'.')[-1]

    @property
    def absolute_path(self):
        if self.is_package():
            return self.abs_path_to_package
        else:
            module_filename = u'%s.py' % self.module_name
            return os.path.join(self.abs_path_to_package, module_filename)

    @property
    def relative_path(self):
        root_path = pkg_resources.resource_filename('reahl.doc.examples', '')
        return self.absolute_path[len(root_path):]

    @property
    def checkout_directory_name(self):
        return self.module_name

    @property
    def exists(self):
        try:
            return pkg_resources.resource_exists('reahl.doc.examples', self.relative_path.replace(os.sep, u'/'))
        except ImportError:
            return False

    @property
    def checkout_dest(self):
        return os.path.join(os.getcwd(), os.path.basename(self.absolute_path))

    @property
    def is_checked_out(self):
        return os.path.exists(self.checkout_dest)

    def check_out(self):
        if self.is_package():
            self.checkout_dir(self.absolute_path, self.checkout_dest)
        else:
            self.sed_file_to(self.absolute_path, self.checkout_dest)

    @property
    def is_i18n_example(self):
        return self.name == u'tutorial.i18nexample'

    def sed_file_to(self, source_filename, dest_filename):
        full_module_name = u'reahl.doc.examples.%s' % self.name
        print dest_filename
        with codecs.open(source_filename, 'r', 'utf-8') as source_file:
            with codecs.open(dest_filename, 'w', 'utf-8') as dest_file:
                for source_line in source_file:
                    if self.is_i18n_example:
                        source_line = source_line.replace(u'Translator(u\'reahl-doc\')', u'Translator(u\'i18nexample\')')
                    dest_file.write(source_line.replace(u'%s.' % full_module_name, u''))

    def checkout_dir(self, source, dest):
        files_to_rename = {u'reahl-doc.po': u'i18nexample.po',
                           u'reahl-doc':    u'i18nexample'}
        for dirpath, dirnames, filenames in os.walk(source):
            relative_dirpath = dirpath[len(source)+1:]
            dest_dirname = os.path.join(dest, relative_dirpath)
            os.mkdir(dest_dirname)
            for filename in [f for f in filenames if not re.match(u'.*(.pyc|~|.mo)$', f)]:
                source_filename = os.path.join(dirpath, filename)
                dest_filename = os.path.join(dest_dirname, filename)
                if self.is_i18n_example and filename in files_to_rename:
                    dest_filename = os.path.join(dest_dirname, files_to_rename[filename])
                if source_filename != os.path.join(source, u'__init__.py'):
                    self.sed_file_to(source_filename, dest_filename)
                        
    def delete(self):
        if os.path.isdir(self.checkout_dest):
            shutil.rmtree(self.checkout_dest)
        elif os.path.isfile(self.checkout_dest):
            os.remove(self.checkout_dest)
           

class ListExamples(WorkspaceCommand):
    """Lists available examples."""
    keyword = 'listexamples'
    def execute(self, options, args):
        for example in Example.get_all():
            print example.name


class GetExample(WorkspaceCommand):
    """Give you a copy of an example."""
    keyword = 'example'
    usage_args = '<example_name>'
    options = [('-f', '--force-overwrite', 
                dict(action='store_true', dest='force', default=False, help='overwrite an existing example if in the way'))]

    def verify_commandline(self, options, args):
        if len(args) != 1:
            self.parser.error('You need to specify one and only one example name as argument')
        if not Example(args[0]).exists:
            self.parser.error('Could not find example %s' % args[0])

    def execute(self, options, args):
        example = Example(args[0])

        if example.is_checked_out:
            if options.force:
                print 'Deleting %s, it is in the way' % os.path.abspath(example.checkout_dest)
                example.delete()
            else:
                print 'First remove %s, it is in the way' % os.path.abspath(example.checkout_dest)
                return 3
        print 'Checking out to %s' % os.path.abspath(example.checkout_dest)
        example.check_out()




from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage

class HelloApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage)
        self.define_view(u'/', title=u'Home')





# In production this has to be set, to the name of the egg of your application:
reahlsystem.root_egg = 'helloapache'   

# If using SQLite:
reahlsystem.connection_uri = 'sqlite:////var/local/helloapache/helloapache.db'  

# If using PostgreSQL:
#reahlsystem.connection_uri = 'postgres://helloapache:helloapache@localhost/helloapache'  



from reahl.tofu import test, Fixture

class SimpleFixture(Fixture):
    def new_name(self):
        return u'John'


@test(SimpleFixture)
def fixture_singletons(fixture):
    """Accessing an attribute on the Fixture always brings back the same 
       object, as created by a similarly named new_ method on the fixture."""

    assert fixture.name == u'John'
    assert fixture.name is fixture.name  # ie, the same object is always returned


# ------- dependent setup objects example

class User(object):
    def __init__(self, name):
        self.name = name


class InterestingFixture(SimpleFixture):
    def new_user(self):
        return User(self.name)


@test(InterestingFixture)
def dependent_setup_objects(fixture):
    """Different attributes on a Fixture can reference one another."""
    assert fixture.user.name is fixture.name


# ------- using new_ methods directly

class MoreInterestingFixture(SimpleFixture):
    def new_user(self, name=None):
        return User(name or self.name)


@test(MoreInterestingFixture)
def bypassing_the_singleton(fixture):
    """new_ methods can be supplied with kwargs in order to create test objects that differ from the default."""
    jane = fixture.new_user(name=u'Jane')
    assert jane.name == u'Jane'
    assert fixture.user is not jane

    other_jane = fixture.new_user(name=u'Jane')
    assert jane is not other_jane








from reahl.tofu import test

from reahl.web_dev.fixtures import WebFixture
from reahl.webdev.tools import Browser, XPath

from reahl.doc.examples.tutorial.componentconfig.componentconfig import AddressBookApp, Address, AddressConfig


class ConfigFixture(WebFixture):
    def new_webapp(self):
        return super(ConfigFixture, self).new_webapp(site_root=AddressBookApp)
        
    def new_existing_address(self):
        address = Address(name=u'John Doe', email_address=u'johndoe@some.org')
        address.save()
        return address

    def new_browser(self):
        return Browser(self.webapp)

    def address_is_listed_as(self, name, email_address):
        return self.browser.is_element_present(XPath.paragraph_containing(u'%s: %s' % (name, email_address)))

    def heading_is_displayed(self):
        return self.browser.is_element_present(XPath.heading_with_text(1, u'Addresses'))


@test(ConfigFixture)
def add_address(fixture):
    """A user can add an address, after which the address is listed."""
    browser = fixture.browser
    
    browser.open(u'/')
    browser.type(XPath.input_labelled(u'Name'), u'John')
    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')

    browser.click(XPath.button_labelled(u'Save'))
    
    assert fixture.address_is_listed_as(u'John', u'johndoe@some.org')


@test(ConfigFixture)
def config_was_read_from_file(fixture):
    """The tests are run with config read from an actual config file, not the default config."""
    assert fixture.context.config.componentconfig.showheader == True


@test(ConfigFixture)
def configurable_heading(fixture):
    """Whether the heading is displayed or not, is configurable."""
    browser = fixture.browser
    
    fixture.context.config.componentconfig.showheader = False
    browser.open(u'/')
    assert not fixture.heading_is_displayed()
    
    fixture.context.config.componentconfig.showheader = True
    browser.open(u'/')
    assert fixture.heading_is_displayed()






from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, TabbedPanel, Tab, P


class TabbedPanelApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')  

        home = self.define_view(u'/', title=u'Tabbed panel demo')
        home.set_slot(u'main', MyTabbedPanel.factory())


class MyTabbedPanel(TabbedPanel):
    def __init__(self, view):
        super(MyTabbedPanel, self).__init__(view, u'mytabbedpanel')
        self.add_tab(Tab(view, u'Tab 1', u'1', P.factory(text=u'A paragraph to give content to the first tab.')))
        self.add_tab(Tab(view, u'Tab 2', u'2', P.factory(text=u'And another ...  to give content to the second tab.')))
        self.add_tab(Tab(view, u'Tab 3', u'3', P.factory(text=u'Something else on the third tab.')))






from reahl.web.fw import Region, Url, Region
from reahl.web.ui import TwoColumnPage, P, Panel, HMenu
from reahl.component.i18n import Translator

_ = Translator(u'reahl-doc')


class SomeContents(Panel):
    def __init__(self, view):
        super(SomeContents, self).__init__(view)

        self.add_child(HMenu.from_languages(view))

        current_url = Url.get_current_url()
        message = _(u'This is a translated string. The current URL is "%s".') % current_url.path
        self.add_child(P(view, text=message))


class TranslatedApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')
        
        home = self.define_view(u'/some_page', title=_(u'Translated example'))
        home.set_slot(u'main', SomeContents.factory())








from nose.tools import istest

import elixir
from reahl.sqlalchemysupport import Session, metadata
from reahl.component.context import ExecutionContext

class Address(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    elixir.using_mapper_options(save_on_init=False)

    email_address = elixir.Field(elixir.UnicodeText)
    name          = elixir.Field(elixir.UnicodeText)

    def save(self):
        Session.add(self)


@istest
def test_model():
    metadata.bind = 'sqlite:///:memory:'
    elixir.setup_all()
    elixir.create_all()

    with ExecutionContext():

        Address(name=u'John', email_address=u'john@world.com').save()
        Address(name=u'Jane', email_address=u'jane@world.com').save()

        addresses = Address.query.all()

        assert addresses[0].name == u'John'
        assert addresses[0].email_address == u'john@world.com'

        assert addresses[1].name == u'Jane'
        assert addresses[1].email_address == u'jane@world.com'





from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, P, YuiGrid, YuiUnit
from reahl.component.modelinterface import exposed, Field, EmailField

some_text = u'Lorem ipsum dolor sit amet, consectetur adipiscing elit.'\
            u' Cras arcu libero, semper rutrum malesuada sit amet, mollis sed dui. '\
            u'Nulla eleifend mollis elit et luctus. Aliquam sollicitudin suscipit '\
            u'mattis. Morbi tincidunt enim non felis tempor rutrum. Donec id mi a '\
            u'neque rutrum dapibus. Nam sed nulla aliquam tellus feugiat mattis '\
            u'nec sit amet arcu. Ut non velit a risus hendrerit facilisis. Proin '\
            u'mattis orci sed sapien malesuada in consectetur eros pellentesque.'

class LayoutApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')  

        home = self.define_view(u'/', title=u'Layout demo')
        home.set_slot(u'main', CommentForm.factory())

        home.set_slot(u'secondary', P.factory(text=u'The secondary column sits on'
                                                   u' the left side of the main column. '+some_text))
        home.set_slot(u'header', P.factory(text=u'This text is located in the header,'
                                                u'which is provided for in a TwoColumnPage. '+some_text))
        home.set_slot(u'footer', P.factory(text=u'The footer spans the bottom of all the '
                                                u'columns on a TwoColumnPage '+ some_text))



class Comment(object):
    @exposed
    def fields(self, fields):
        fields.email_address = EmailField(label=u'Email address', required=True)
        fields.text = Field(label=u'Comment text')

class CommentForm(Form):
    def __init__(self, view):
        super(CommentForm, self).__init__(view, u'myform')

        comment = Comment()
        self.add_child( LabelledBlockInput(TextInput(self, comment.fields.email_address)) )
        self.add_child( LabelledBlockInput(TextInput(self, comment.fields.text)) )

        row = self.add_child(YuiGrid(view))
        [left_unit, right_unit] = row.add_children([YuiUnit(view, first=True), YuiUnit(view)])

        left_unit.add_child( P(view, text=u'This is in the left block of the row') ) 
        right_unit.add_child( P(view, text=u'This is in the right block of the row') ) 




# To run this test do:
# nosetests -F reahl.webdev.fixtures:BrowserSetup -s --nologcapture reahl/doc_dev/tutorialtests/pagertests.py
#



from reahl.tofu import test
from reahl.web_dev.fixtures import WebFixture
from reahl.webdev.tools import XPath
from reahl.doc.examples.tutorial.pager.pager import AddressBookApp


class PagingFixture(WebFixture):
    def new_browser(self):
        return self.driver_browser
        
    def new_webapp(self):
        return super(PagingFixture, self).new_webapp(site_root=AddressBookApp, enable_js=True)

    def is_email_listed(self, email):
        return self.browser.is_element_present(XPath.paragraph_containing(email))


@test(PagingFixture)
def paging(fixture):
    """Clicking on a different page in the pager changes the addresses listed without triggering a page load."""

    fixture.reahl_server.set_app(fixture.webapp)
    browser = fixture.browser

    browser.open(u'/')
    
    assert fixture.is_email_listed(u'friend0@some.org')
    assert not fixture.is_email_listed(u'friend9@some.org')

    with browser.no_page_load_expected():
        browser.click(XPath.link_with_text(u'2'))

    assert not fixture.is_email_listed(u'friend0@some.org')
    assert fixture.is_email_listed(u'friend9@some.org')










from reahl.tofu import test

from reahl.web_dev.fixtures import WebFixture
from reahl.webdev.tools import Browser, XPath
from reahl.sqlalchemysupport import Session

from reahl.doc.examples.tutorial.jobs.jobs import AddressBookApp, Address


class JobsFixture(WebFixture):
    def new_webapp(self):
        return super(JobsFixture, self).new_webapp(site_root=AddressBookApp)
        
    def new_existing_address(self):
        address = Address(name=u'John Doe', email_address=u'johndoe@some.org')
        address.save()
        return address

    def new_browser(self):
        return Browser(self.webapp)

    def address_is_listed_as(self, name, email_address, is_new):
        new = u' (new)' if is_new else u''
        return self.browser.is_element_present(XPath.paragraph_containing(u'%s: %s%s' % (name, email_address, new)))


@test(JobsFixture)
def add_address(fixture):
    """A user can add an address, after which the address is listed."""
    browser = fixture.browser
    
    browser.open(u'/')
    browser.type(XPath.input_labelled(u'Name'), u'John')
    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')

    browser.click(XPath.button_labelled(u'Save'))
    
    assert fixture.address_is_listed_as(u'John', u'johndoe@some.org', True)


@test(JobsFixture)
def daily_maintenance(fixture):
    """When daily maintenance is run, all addresses are set to be old."""
    browser = fixture.browser

    fixture.existing_address
    Session.flush()
    assert fixture.existing_address.added_today

    fixture.context.system_control.do_daily_maintenance()

    assert not fixture.existing_address.added_today






from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, Button, LabelledBlockInput
from reahl.component.modelinterface import exposed, EmailField, secured, Event, Action, PasswordField

class AccessApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')  

        home = self.define_view(u'/', title=u'Access control demo')
        home.set_slot(u'main', CommentForm.factory())


class Comment(object):
    def always_allowed(self):
        return True

    def never_allowed(self):
        return False

    @exposed
    def fields(self, fields):
        fields.greyed_out_field = EmailField(label=u'Greyed out',
                                             default=u'some default value',
                                             readable=Action(self.always_allowed),
                                             writable=Action(self.never_allowed))

    @exposed
    def events(self, events):
        events.greyed_out_event = Event(label=u'Greyed out button', action=Action(self.do_something))

    @secured(read_check=always_allowed, write_check=never_allowed)
    def do_something(self): 
        pass


class CommentForm(Form):
    def __init__(self, view):
        super(CommentForm, self).__init__(view, u'myform')

        comment = Comment()
        self.add_child( LabelledBlockInput(TextInput(self, comment.fields.greyed_out_field)) )

        self.define_event_handler(comment.events.greyed_out_event)
        self.add_child( Button(self, comment.events.greyed_out_event) )











from datetime import datetime

import elixir



from reahl.sqlalchemysupport import Session, metadata

from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, P, H, InputGroup
from reahl.component.modelinterface import exposed, EmailField, Field, Event, Action
from reahl.component.migration import Migration

from sqlalchemy import Column, DateTime
from alembic import op

class AddDate(Migration):
    version = u'0.1'
    def upgrade(self):
        print 'executing AddDate (upgrade)'
        op.add_column(u'migration_address', Column('added_date', DateTime))
            
    def upgrade_cleanup(self):
        print 'executing AddDate (upgrade_cleanup)'



class AddressBookApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')
        find = self.define_view(u'/', title=u'Addresses')
        find.set_slot(u'main', AddressBookPanel.factory())


class AddressBookPanel(Panel):
    def __init__(self, view):
        super(AddressBookPanel, self).__init__(view)

        self.add_child(H(view, 1, text=u'Addresses'))
        
        for address in Address.query.all():
            self.add_child(AddressBox(view, address))

        self.add_child(AddAddressForm(view))


class AddAddressForm(Form):
    def __init__(self, view):
        super(AddAddressForm, self).__init__(view, u'add_form')

        new_address = Address()

        grouped_inputs = self.add_child(InputGroup(view, label_text=u'Add an address'))
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_address.fields.name)) )
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_address.fields.email_address)) )

        self.define_event_handler(new_address.events.save)
        grouped_inputs.add_child(Button(self, new_address.events.save))


class AddressBox(Panel):
    def __init__(self, view, address):
        super(AddressBox, self).__init__(view)
        self.add_child(P(view, text=u'%s: %s (%s)' % (address.name, address.email_address, address.added_date)))


class Address(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    elixir.using_mapper_options(save_on_init=False)
    
    email_address = elixir.Field(elixir.UnicodeText)
    name          = elixir.Field(elixir.UnicodeText)
#    added_date    = elixir.Field(elixir.DateTime)
    added_date    = u'TODO'

    @exposed
    def fields(self, fields):
        fields.name = Field(label=u'Name', required=True)
        fields.email_address = EmailField(label=u'Email', required=True)

    def save(self):
        self.added_date = datetime.now()
        Session.add(self)
        
    @exposed
    def events(self, events):
        events.save = Event(label=u'Save', action=Action(self.save))




# Copyright 2012, 2013 Reahl Software Services (Pty) Ltd. All rights reserved.
#
#    This file is part of Reahl.
#
#    Reahl is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation; version 3 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Panel, Form, TextInput, Button, Form, \
                          LabelOverInput, CueInput, CheckboxInput, TextInput, \
                          PasswordInput, Button, LabelledInlineInput, LabelledBlockInput, P,\
                          TextArea, SelectInput, RadioButtonInput

from reahl.component.modelinterface import exposed, Field, BooleanField, ChoiceField, Choice, ChoiceGroup, \
                                           IntegerField, Event, MultiChoiceField, Action, DateField

class BasicHTMLInputsApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')  

        home = self.define_view(u'/', title=u'Basic HTML Inputs demo')
        home.set_slot(u'main', ExampleForm.factory(u'myform'))


class ModelObject(object):
    multi_choice_field = [1, 3]
    choice_field = 2
    @exposed
    def fields(self, fields):
        fields.text_input_field = Field(label=u'A TextInput')
        fields.password_field = Field(label=u'A PasswordInput')
        fields.text_area_field = Field(label=u'A TextArea')
        fields.label_over_field = Field(label=u'A TextInput in a LabelOverInput')
        fields.labelled_inline_field = Field(label=u'A TextInput in a LabelledInlineInput')
        fields.cue_field = Field(label=u'A TextInput in a CueInput')
        fields.choice_field = ChoiceField([Choice(False, BooleanField(label=u'None selected')),
                                           ChoiceGroup(u'Numbers', [Choice(1, IntegerField(label=u'One')), 
                                                                    Choice(2, IntegerField(label=u'Two')), 
                                                                    Choice(3, IntegerField(label=u'Three'))]),
                                           ChoiceGroup(u'Colours', [Choice(u'r', Field(label=u'Red')),
                                                                    Choice(u'g', Field(label=u'Green'))])
                                          ], label=u'A SelectInput')
        fields.multi_choice_field = MultiChoiceField([Choice(1, IntegerField(label=u'One')), 
                                                      Choice(2, IntegerField(label=u'Two')), 
                                                      Choice(3, IntegerField(label=u'Three'))], 
                                                      label=u'A SelectInput allowing multiple choices')
        fields.boolean_field = BooleanField(label=u'A CheckboxInput')
        fields.radio_choice_field = ChoiceField([Choice(1, IntegerField(label=u'One')), 
                                                 Choice(2, IntegerField(label=u'Two')), 
                                                 Choice(3, IntegerField(label=u'Three'))],
                                                label=u'A RadioButtonInput')
        fields.fuzzy_date_field = DateField(label=u'A fuzzy TextInput for a Date')
        
    @exposed
    def events(self, events):
        events.do_something = Event(label=u'A Button')
    

class ExampleForm(Form):
    def __init__(self, view, event_channel_name):
        super(ExampleForm, self).__init__(view, event_channel_name)
        
        model_object = ModelObject()
        self.add_child(LabelledBlockInput(TextInput(self, model_object.fields.text_input_field)))
        self.add_child(LabelledBlockInput(CheckboxInput(self, model_object.fields.boolean_field)))
        self.add_child(LabelledBlockInput(PasswordInput(self, model_object.fields.password_field)))
        self.add_child(LabelledBlockInput(TextArea(self, model_object.fields.text_area_field, rows=5, columns=60)))
        self.add_child(LabelledBlockInput(SelectInput(self, model_object.fields.choice_field)))
        self.add_child(LabelledBlockInput(SelectInput(self, model_object.fields.multi_choice_field)))
        self.add_child(LabelledBlockInput(RadioButtonInput(self, model_object.fields.radio_choice_field)))
        self.add_child(LabelledBlockInput(TextInput(self, model_object.fields.fuzzy_date_field, fuzzy=True)))

        self.add_child(LabelOverInput(TextInput(self, model_object.fields.label_over_field)))
        self.add_child(LabelledInlineInput(TextInput(self, model_object.fields.labelled_inline_field)))
        self.add_child(CueInput(TextInput(self, model_object.fields.cue_field), P(view, text=u'This is a cue')))


        self.define_event_handler(model_object.events.do_something)
        self.add_child(Button(self, model_object.events.do_something))
        




import elixir
from sqlalchemy.orm.exc import NoResultFound

from reahl.sqlalchemysupport import Session, metadata
from reahl.elixirsupport import session_scoped

from reahl.web.fw import Region, UrlBoundView, CannotCreate
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, P, H, InputGroup, HMenu,\
                         PasswordInput, ErrorFeedbackMessage, VMenu, Slot, MenuItem, A, Widget, SelectInput, CheckboxInput
from reahl.systemaccountmodel import AccountManagementInterface, EmailAndPasswordSystemAccount, UserSession
from reahl.component.modelinterface import exposed, IntegerField, BooleanField, Field, EmailField, Event, Action, Choice, ChoiceField



class Address(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    elixir.using_mapper_options(save_on_init=False)
    
    address_book  = elixir.ManyToOne(u'reahl.doc.examples.tutorial.access.access.AddressBook')
    email_address = elixir.Field(elixir.UnicodeText)
    name          = elixir.Field(elixir.UnicodeText)

    @classmethod
    def by_id(cls, address_id, exception_to_raise):
        addresses = Address.query.filter_by(id=address_id)
        if addresses.count() != 1:
            raise exception_to_raise
        return addresses.one()

    @exposed
    def fields(self, fields):
        fields.name = Field(label=u'Name', required=self.can_be_added(), writable=Action(self.can_be_added))
        fields.email_address = EmailField(label=u'Email', required=True, writable=Action(self.can_be_edited))

    @exposed(u'save', u'update', u'edit')
    def events(self, events):
        events.save = Event(label=u'Save', action=Action(self.save))
        events.update = Event(label=u'Update')
        events.edit = Event(label=u'Edit', writable=Action(self.can_be_edited))

    def save(self):
        Session.add(self)

    def can_be_edited(self):
        current_account = UserSession.for_current_session().account
        return self.address_book.can_be_edited_by(current_account)

    def can_be_added(self):
        current_account = UserSession.for_current_session().account
        return self.address_book.can_be_added_to_by(current_account)


class AddressBook(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)

    owner      = elixir.ManyToOne(EmailAndPasswordSystemAccount, required=True)

    @classmethod
    def by_id(cls, address_book_id, exception_to_raise):
        address_books = AddressBook.query.filter_by(id=address_book_id)
        if address_books.count() != 1:
            raise exception_to_raise
        return address_books.one()
    
    @classmethod
    def owned_by(cls, account):
        return cls.query.filter_by(owner=account)
        
    @classmethod
    def address_books_visible_to(cls, account):
        visible_books = cls.query.join(Collaborator).filter(Collaborator.account == account).all()
        visible_books.extend(cls.owned_by(account))
        return visible_books

    @exposed
    def fields(self, fields):
        collaborators = [Choice(i.id, IntegerField(label=i.email)) for i in EmailAndPasswordSystemAccount.query.all()]
        fields.chosen_collaborator = ChoiceField(collaborators, label=u'Choose collaborator')
        fields.may_edit_address = BooleanField(label=u'May edit existing addresses')
        fields.may_add_address = BooleanField(label=u'May add new addresses')

    @exposed(u'add_collaborator')
    def events(self, events):
        events.add_collaborator = Event(label=u'Share', action=Action(self.add_collaborator))

    def add_collaborator(self):
        chosen_account = EmailAndPasswordSystemAccount.query.filter_by(id=self.chosen_collaborator).one()
        self.allow(chosen_account, can_add_addresses=self.may_add_address, can_edit_addresses=self.may_edit_address)

    # See https://groups.google.com/forum/?fromgroups=#!topic/sqlelixir/ZlR9Kvcor6Q
    #    addresses  = elixir.OneToMany(Address)
    @property
    def addresses(self):
        return Address.query.filter_by(address_book=self).all()
    collaborators = elixir.OneToMany(u'reahl.doc.examples.tutorial.access.access.Collaborator', lazy=u'dynamic')

    @property
    def display_name(self):
        return u'Address book of %s' % self.owner.email

    def allow(self, account, can_add_addresses=False, can_edit_addresses=False):
        Collaborator.query.filter_by(address_book=self, account=account).delete()
        Collaborator(address_book=self, account=account,
            can_add_addresses=can_add_addresses,
            can_edit_addresses=can_edit_addresses)

    def can_be_edited_by(self, account):
        if account is self.owner: 
            return True
            
        collaborator = self.get_collaborator(account)
        return (collaborator and collaborator.can_edit_addresses) or self.can_be_added_to_by(account)

    def can_be_added_to_by(self, account):
        if account is self.owner: 
            return True

        collaborator = self.get_collaborator(account)
        return collaborator and collaborator.can_add_addresses
        
    def can_be_added_to(self):
        account = UserSession.for_current_session().account
        return self.can_be_added_to_by(account)

    def collaborators_can_be_added_by(self, account):
        return self.owner is account

    def collaborators_can_be_added(self):
        account = UserSession.for_current_session().account
        return self.collaborators_can_be_added_by(account)

    def is_visible_to(self, account):
        return self in self.address_books_visible_to(account)

    def is_visible(self):
        account = UserSession.for_current_session().account
        return self.is_visible_to(account)
        
    def get_collaborator(self, account):            
        collaborators = self.collaborators.filter_by(account=account)
        count = collaborators.count()
        if count == 1:
            return collaborators.one()
        if count > 1:
            raise ProgrammerError('There can be only one Collaborator per account. Here is more than one.')
        return None


class Collaborator(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)

    account = elixir.ManyToOne(EmailAndPasswordSystemAccount)
    can_add_addresses = elixir.Field(elixir.Boolean, default=False)
    can_edit_addresses = elixir.Field(elixir.Boolean, default=False)

    address_book = elixir.ManyToOne(AddressBook)


class AddressAppPage(TwoColumnPage):
    def __init__(self, view, home_bookmark):
        super(AddressAppPage, self).__init__(view, style=u'basic')

        user_session = UserSession.for_current_session()
        if user_session.is_logged_in():
            logged_in_as = user_session.account.email
        else:
            logged_in_as = u'Not logged in'

        self.header.add_child(P(view, text=logged_in_as))
        self.header.add_child(HMenu.from_bookmarks(view, [home_bookmark]))


class LoginForm(Form):
    def __init__(self, view, accounts):
        super(LoginForm, self).__init__(view, u'login')
        
        if self.exception:
            self.add_child(ErrorFeedbackMessage(view, self.exception.as_user_message()))

        self.add_child(LabelledBlockInput(TextInput(self, accounts.fields.email)))
        self.add_child(LabelledBlockInput(PasswordInput(self, accounts.fields.password)))

        self.define_event_handler(accounts.events.login_event)
        self.add_child(Button(self, accounts.events.login_event))


class LogoutForm(Form):
    def __init__(self, view, accounts):
        super(LogoutForm, self).__init__(view, u'logout')
        self.define_event_handler(accounts.events.log_out_event)
        self.add_child(Button(self, accounts.events.log_out_event))


class HomePageWidget(Widget):
    def __init__(self, view, address_book_app):
        super(HomePageWidget, self).__init__(view)
        accounts = AccountManagementInterface.for_current_session()
        user_session = UserSession.for_current_session()
        if user_session.is_logged_in():
            self.add_child(AddressBookList(view, address_book_app))
            self.add_child(LogoutForm(view, accounts))
        else:
            self.add_child(LoginForm(view, accounts))


class AddressBookList(Panel):
    def __init__(self, view, address_book_app):
        super(AddressBookList, self).__init__(view)

        current_account = UserSession.for_current_session().account
        address_books = [book for book in AddressBook.address_books_visible_to(current_account)]
        bookmarks = [address_book_app.get_address_book_bookmark(address_book, description=address_book.display_name)
                     for address_book in address_books]

        for bookmark in bookmarks:
            p = self.add_child(P(view))
            p.add_child(A.from_bookmark(view, bookmark))
        

class AddressBookPanel(Panel):
    def __init__(self, view, address_book, address_book_app):
    	self.address_book = address_book
        super(AddressBookPanel, self).__init__(view)
        
        self.add_child(H(view, 1, text=u'Addresses in %s' % address_book.display_name))
        self.add_child(HMenu.from_bookmarks(view, self.menu_bookmarks(address_book_app)))
        self.add_children([AddressBox(view, address) for address in address_book.addresses])

    def menu_bookmarks(self, address_book_app):
        return [address_book_app.get_add_address_bookmark(self.address_book), 
                address_book_app.get_add_collaborator_bookmark(self.address_book)]


class EditAddressForm(Form):
    def __init__(self, view, address):
        super(EditAddressForm, self).__init__(view, u'edit_form')

        grouped_inputs = self.add_child(InputGroup(view, label_text=u'Edit address'))
        grouped_inputs.add_child(LabelledBlockInput(TextInput(self, address.fields.name)))
        grouped_inputs.add_child(LabelledBlockInput(TextInput(self, address.fields.email_address)))

        grouped_inputs.add_child(Button(self, address.events.update.with_arguments(address_book_id=address.address_book.id)))


class AddAddressForm(Form):
    def __init__(self, view, address_book):
        super(AddAddressForm, self).__init__(view, u'add_form')

        new_address = Address(address_book=address_book)

        grouped_inputs = self.add_child(InputGroup(view, label_text=u'Add an address'))
        grouped_inputs.add_child(LabelledBlockInput(TextInput(self, new_address.fields.name)))
        grouped_inputs.add_child(LabelledBlockInput(TextInput(self, new_address.fields.email_address)))

        grouped_inputs.add_child(Button(self, new_address.events.save.with_arguments(address_book_id=address_book.id)))


class AddressBox(Form):
    def __init__(self, view, address):
        form_name = u'address_%s' % address.id
        super(AddressBox, self).__init__(view, form_name)

        par = self.add_child(P(view, text=u'%s: %s ' % (address.name, address.email_address)))
        par.add_child(Button(self, address.events.edit.with_arguments(address_id=address.id)))


class AddressBookView(UrlBoundView):
    def assemble(self, address_book_id=None, address_book_app=None):
        address_book = AddressBook.by_id(address_book_id, CannotCreate())

        self.title = address_book.display_name
        self.set_slot(u'main', AddressBookPanel.factory(address_book, address_book_app))
        self.read_check = address_book.is_visible


class AddAddressView(UrlBoundView):
    def assemble(self, address_book_id=None):
        address_book = AddressBook.by_id(address_book_id, CannotCreate())

        self.title = u'Add to %s' % address_book.display_name
        self.set_slot(u'main', AddAddressForm.factory(address_book))
        self.write_check = address_book.can_be_added_to


class AddCollaboratorForm(Form):
    def __init__(self, view, address_book):
        super(AddCollaboratorForm, self).__init__(view, u'add_collaborator_form')

        grouped_inputs = self.add_child(InputGroup(view, label_text=u'Add a collaborator'))
        grouped_inputs.add_child(LabelledBlockInput(SelectInput(self, address_book.fields.chosen_collaborator)))

        rights_inputs = grouped_inputs.add_child(InputGroup(view, label_text=u'Rights'))
        rights_inputs.add_child(LabelledBlockInput(CheckboxInput(self, address_book.fields.may_edit_address)))
        rights_inputs.add_child(LabelledBlockInput(CheckboxInput(self, address_book.fields.may_add_address)))

        grouped_inputs.add_child(Button(self, address_book.events.add_collaborator.with_arguments(address_book_id=address_book.id)))


class AddCollaboratorView(UrlBoundView):
    def assemble(self, address_book_id=None):
        address_book = AddressBook.by_id(address_book_id, CannotCreate())

        self.title = u'Add collaborator to %s' % address_book.display_name
        self.set_slot(u'main', AddCollaboratorForm.factory(address_book))
        self.read_check = address_book.collaborators_can_be_added


class EditAddressView(UrlBoundView):
    def assemble(self, address_id=None):
        address = Address.by_id(address_id, CannotCreate())

        self.title = u'Edit Address for %s' % address.name
        self.set_slot(u'main', EditAddressForm.factory(address))
        self.read_check = address.can_be_edited


class AddressBookApp(Region):
    def assemble(self):

        home = self.define_view(u'/', title=u'Address books')
        home.set_slot(u'main', HomePageWidget.factory(self))
      
        self.address_book_page = self.define_view(u'/address_book', view_class=AddressBookView, 
                                                  address_book_id=IntegerField(required=True),
                                                  address_book_app=self)
        self.add_address_page = self.define_view(u'/add_address', view_class=AddAddressView, 
                                                 address_book_id=IntegerField(required=True))

        edit_address_page = self.define_view(u'/edit_address', view_class=EditAddressView,
                                             address_id=IntegerField(required=True))

        self.add_collaborator_page = self.define_view(u'/add_collaborator', view_class=AddCollaboratorView, 
                                                     address_book_id=IntegerField(required=True))
        
        self.define_transition(Address.events.save, self.add_address_page, self.address_book_page)
        self.define_transition(Address.events.edit, self.address_book_page, edit_address_page)
        self.define_transition(Address.events.update, edit_address_page, self.address_book_page)
        self.define_transition(AddressBook.events.add_collaborator, self.add_collaborator_page, self.address_book_page)

        self.define_main_window(AddressAppPage, home.as_bookmark(self))

    def get_address_book_bookmark(self, address_book, description=None):
        return self.address_book_page.as_bookmark(self, description=description, address_book_id=address_book.id)

    def get_add_address_bookmark(self, address_book, description=u'Add address'):
        return self.add_address_page.as_bookmark(self, description=description, address_book_id=address_book.id)
        
    def get_add_collaborator_bookmark(self, address_book, description=u'Add collaborator'):
        return self.add_collaborator_page.as_bookmark(self, description=description, address_book_id=address_book.id)









from nose.tools import istest
from reahl.tofu import expected, NoException

import elixir
from reahl.sqlalchemysupport import Session, metadata
from reahl.component.modelinterface import exposed, EmailField, Field, Event, Action
from reahl.component.context import ExecutionContext


class Address(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    elixir.using_mapper_options(save_on_init=False)

    email_address = elixir.Field(elixir.UnicodeText)
    name          = elixir.Field(elixir.UnicodeText)

    def save(self):
        Session.add(self)

    @exposed
    def fields(self, fields):
        fields.name = Field(label=u'Name', required=True)
        fields.email_address = EmailField(label=u'Email', required=True)

    @exposed
    def events(self, events):
        events.save = Event(label=u'Save', action=Action(self.save))


@istest
def test_reahl_additions():

    metadata.bind = 'sqlite:///:memory:'
    elixir.setup_all()
    elixir.create_all()

    with ExecutionContext():

        address = Address()
        email_field = address.fields.email_address
        
        # While a programmer would not usually write code like this,
        # it is useful to show how the framework can use Fields and Events
        # to obtain more information about a certain Field/Event:
        assert email_field.label == u'Email'

        # Fields are used (amongst other things) to validate user input:
        with expected(Exception):
            email_field.from_input(u'invalid email address')

        with expected(NoException):
            assert address.email_address == None
            email_field.from_input(u'valid@email.com')
            assert address.email_address == u'valid@email.com'

        # After input was given, the field is set on the object it belongs to:
        # (The value set is a marshalled version of the user input. In this case it is just
        #  a string again, but it could have been, for example an EmailAddress object,
        #  and Integer, or a Date.)
        assert address.email_address == u'valid@email.com'




# To run this test do:
# nosetests -F reahl.webdev.fixtures:BrowserSetup -s --nologcapture reahl/doc_dev/tutorialtests/accesstests2.py
#
# To set up a demo database for playing with, do:
# nosetests -F reahl.webdev.fixtures:BrowserSetup --with-setup-fixture=reahl.doc_dev.tutorialtests.accesstests2:DemoFixture -s --nologcapture



from reahl.tofu import test, set_up
from reahl.web_dev.fixtures import WebFixture
from reahl.webdev.tools import Browser, XPath
from reahl.sqlalchemysupport import Session
from reahl.systemaccountmodel import EmailAndPasswordSystemAccount

from reahl.doc.examples.tutorial.access2.access2 import AddressBookApp, AddressBook, Address


class AccessFixture(WebFixture):
    def new_browser(self):
        return Browser(self.new_webapp(site_root=AddressBookApp))
        
    password = u'bobbejaan'

    def new_account(self, email=u'johndoe@some.org'):
        account = EmailAndPasswordSystemAccount(email=email)
        account.set_new_password(account.email, self.password)
        account.activate()
        return account

    def new_address_book(self, owner=None):
        owner = owner or self.account
        return AddressBook(owner=owner)

    def new_other_account(self):
        return self.new_account(email=u'other@some.org')
        
    def new_other_address_book(self):
        return self.new_address_book(owner=self.other_account)

    def is_on_address_book_page_of(self, email):
        return self.browser.title == u'Address book of %s' % email


class DemoFixture(AccessFixture):
    commit=True
    @set_up
    def do_demo_setup(self):
        self.address_book
        john = self.account
        jane = self.new_account(email=u'janedoe@some.org')
        jane_book = self.new_address_book(owner=jane)
        someone = self.new_account(email=u'someone@some.org')
        someone_book = self.new_address_book(owner=someone)
        someone_else = self.new_account(email=u'someoneelse@some.org')
        someone_else_book = self.new_address_book(owner=someone_else)

        jane_book.allow(john, can_add_addresses=True, can_edit_addresses=True)
        someone_book.allow(john, can_add_addresses=False, can_edit_addresses=True)
        someone_else_book.allow(john, can_add_addresses=False, can_edit_addresses=False)
        
        Address(address_book=jane_book, email_address=u'friend1@some.org', name=u'Friend1').save()
        Address(address_book=jane_book, email_address=u'friend2@some.org', name=u'Friend2').save()
        Address(address_book=jane_book, email_address=u'friend3@some.org', name=u'Friend3').save()
        Address(address_book=jane_book, email_address=u'friend4@some.org', name=u'Friend4').save()

        Address(address_book=someone_book, email_address=u'friend11@some.org', name=u'Friend11').save()
        Address(address_book=someone_book, email_address=u'friend12@some.org', name=u'Friend12').save()
        Address(address_book=someone_book, email_address=u'friend13@some.org', name=u'Friend13').save()
        Address(address_book=someone_book, email_address=u'friend14@some.org', name=u'Friend14').save()

        Address(address_book=someone_else_book, email_address=u'friend21@some.org', name=u'Friend21').save()
        Address(address_book=someone_else_book, email_address=u'friend22@some.org', name=u'Friend22').save()
        Address(address_book=someone_else_book, email_address=u'friend23@some.org', name=u'Friend23').save()
        Address(address_book=someone_else_book, email_address=u'friend24@some.org', name=u'Friend24').save()
        Session.flush()




@test(AccessFixture)
def separate_address_books(fixture):
    """An Address is created in a particular AddressBook, which is owned by a SystemAccount."""

    account = fixture.account
    address_book = fixture.address_book
    other_address_book = fixture.other_address_book

    # AddressBooks are owned
    address_book.owner is account
    other_address_book.owner is fixture.other_account

    # Addresses live in specific AddressBooks
    assert address_book.addresses == []
    assert other_address_book.addresses == []

    address1 = Address(address_book=address_book, email_address=u'friend1@some.org', name=u'Friend1')
    address2 = Address(address_book=address_book, email_address=u'friend2@some.org', name=u'Friend2')

    address3 = Address(address_book=other_address_book, email_address=u'friend3@some.org', name=u'Friend3')

    for address in [address1, address2, address3]:
        address.save()

    assert address_book.addresses == [address1, address2]
    assert other_address_book.addresses == [address3]


@test(AccessFixture)
def collaborators(fixture):
    """A particular SystemAccount can see its own AddressBooks as well as all the AddressBooks
       it is explicitly allowed to see, but no other AddressBooks."""
    
    account = fixture.account
    address_book = fixture.address_book
    other_address_book = fixture.other_address_book

    unrelated_account = fixture.new_account(email=u'unrelated@some.org')
    unrelated_address_book = fixture.new_address_book(owner=unrelated_account)
    
    other_address_book.allow(account)
    
    # Checks to see whether an AddressBook is visible
    assert address_book.is_visible_to(account)
    assert other_address_book.is_visible_to(account)
    assert not unrelated_address_book.is_visible_to(account)

    # Getting a list of visible AddressBooks (for populating the screen)
    books = AddressBook.address_books_visible_to(account)
    assert set(books) == set([address_book, other_address_book])


@test(AccessFixture)
def collaborator_rights(fixture):
    """When allowing an account to see another's AddressBook, the rights it has to the AddressBook are specified."""

    account = fixture.account
    address_book = fixture.address_book
    other_address_book = fixture.other_address_book

    # Case: defaults
    other_address_book.allow(account)
    assert not other_address_book.can_be_edited_by(account)
    assert not other_address_book.can_be_added_to_by(account)

    # Case: rights specified
    other_address_book.allow(account, can_edit_addresses=True, can_add_addresses=True)
    assert other_address_book.can_be_edited_by(account)
    assert other_address_book.can_be_added_to_by(account)


@test(AccessFixture)
def adding_collaborators(fixture):
    """The owner of an AddressBook may add collaborators to it."""

    account = fixture.account
    address_book = fixture.address_book
    other_address_book = fixture.other_address_book

    assert address_book.collaborators_can_be_added_by(account)
    assert not other_address_book.collaborators_can_be_added_by(account)


@test(AccessFixture)
def logging_in(fixture):
    """A user first sees only a login screen on the home page; after logging in, 
       all the address books visible to the user appear."""
    browser = fixture.browser
    account = fixture.account
    address_book = fixture.address_book

    other_address_book = fixture.other_address_book
    other_address_book.allow(account)

    browser.open(u'/')
    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.type(XPath.input_labelled(u'Password'), fixture.password)
    browser.click(XPath.button_labelled(u'Log in'))
    
    assert browser.is_element_present(XPath.link_with_text(u'Address book of johndoe@some.org'))
    assert browser.is_element_present(XPath.link_with_text(u'Address book of other@some.org'))


@test(AccessFixture)
def edit_and_add_own(fixture):
    """The owner of an AddressBook can add and edit Addresses to the owned AddressBook."""
    browser = fixture.browser
    account = fixture.account
    address_book = fixture.address_book

    fixture.log_in(browser=browser, system_account=account)
    browser.open(u'/')
    
    browser.click(XPath.link_with_text(u'Address book of johndoe@some.org'))
    
    # add
    browser.click(XPath.link_with_text(u'Add address'))
    browser.type(XPath.input_labelled(u'Name'), u'Someone')
    browser.type(XPath.input_labelled(u'Email'), u'someone@some.org')
    browser.click(XPath.button_labelled(u'Save'))

    assert browser.is_element_present(XPath.paragraph_containing(u'Someone: someone@some.org'))

    # edit
    browser.click(XPath.button_labelled(u'Edit'))
    browser.type(XPath.input_labelled(u'Name'), u'Else')
    browser.type(XPath.input_labelled(u'Email'), u'else@some.org')
    browser.click(XPath.button_labelled(u'Update'))

    assert browser.is_element_present(XPath.paragraph_containing(u'Else: else@some.org'))
    

@test(AccessFixture)
def add_collaborator(fixture):
    """A user may add other users as collaborators to his address book, specifying the privileges in the process."""   
    browser = fixture.browser
    account = fixture.account
    address_book = fixture.address_book

    other_address_book = fixture.other_address_book
    other_account = fixture.other_account

    fixture.log_in(browser=browser, system_account=account)
    browser.open(u'/')

    assert address_book not in AddressBook.address_books_visible_to(other_account)
    assert not address_book.can_be_edited_by(other_account)
    assert not address_book.can_be_added_to_by(other_account)

    browser.click(XPath.link_with_text(u'Address book of johndoe@some.org'))
    browser.click(XPath.link_with_text(u'Add collaborator'))

    browser.select(XPath.select_labelled(u'Choose collaborator'), u'other@some.org')
    browser.click(XPath.input_labelled(u'May add new addresses'))
    browser.click(XPath.input_labelled(u'May edit existing addresses'))

    browser.click(XPath.button_labelled(u'Share'))

    assert fixture.is_on_address_book_page_of(u'johndoe@some.org')

    assert address_book in AddressBook.address_books_visible_to(other_account)
    assert address_book.can_be_edited_by(other_account)
    assert address_book.can_be_added_to_by(other_account)







from reahl.tofu import test

from reahl.web_dev.fixtures import WebFixture
from reahl.webdev.tools import Browser, XPath

from reahl.doc.examples.tutorial.parameterised2.parameterised2 import AddressBookApp, Address


class AddressAppFixture(WebFixture):
    def new_browser(self):
        return Browser(self.new_webapp(site_root=AddressBookApp))
        
    def new_existing_address(self):
        address = Address(name=u'John Doe', email_address=u'johndoe@some.org')
        address.save()
        return address

    def is_on_home_page(self):
        return self.browser.title == u'Addresses'
        
    def is_on_add_page(self):
        return self.browser.title == u'Add an address'

    def is_on_edit_page_for(self, address):
        return self.browser.title == u'Edit %s' % address.name

    def address_is_listed_as(self, name, email_address):
        return self.browser.is_element_present(XPath.paragraph_containing(u'%s: %s' % (name, email_address)))


@test(AddressAppFixture)
def adding_an_address(fixture):
    """To add a new address, a user clicks on "Add Address" link on the menu, then supplies the 
       information for the new address and clicks the Save button. Upon success addition of the
       address, the user is returned to the home page where the new address is now listed."""

    browser = fixture.browser

    browser.open(u'/')
    browser.click(XPath.link_with_text(u'Add an address'))

    assert fixture.is_on_add_page()
    browser.type(XPath.input_labelled(u'Name'), u'John Doe')
    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.click(XPath.button_labelled(u'Save'))
    
    assert fixture.is_on_home_page()
    assert fixture.address_is_listed_as(u'John Doe', u'johndoe@some.org') 

@test(AddressAppFixture)
def editing_an_address(fixture):
    """To edit an existing address, a user clicks on the "Edit" button next to the chosen Address
       on the "Addresses" page. The user is then taken to an "Edit" View for the chosen Address and 
       can change the name or email address. Upon clicking the "Update" Button, the user is sent back 
       to the "Addresses" page where the changes are visible."""

    browser = fixture.browser
    existing_address = fixture.existing_address
    
    browser.open(u'/')
    browser.click(XPath.button_labelled(u'Edit'))

    assert fixture.is_on_edit_page_for(existing_address)
    browser.type(XPath.input_labelled(u'Name'), u'John Doe-changed')
    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.changed.org')
    browser.click(XPath.button_labelled(u'Update'))
    
    assert fixture.is_on_home_page()
    assert fixture.address_is_listed_as(u'John Doe-changed', u'johndoe@some.changed.org') 
                





from reahl.tofu import test, set_up

from reahl.web_dev.fixtures import WebFixture
from reahl.webdev.tools import Browser, XPath

from reahl.doc.examples.tutorial.login1.login1 import LoginApp
from reahl.systemaccountmodel import EmailAndPasswordSystemAccount


class LoginFixture(WebFixture):
    def new_browser(self):
        return Browser(self.new_webapp(site_root=LoginApp))
        
    password = u'bobbejaan'

    def new_account(self):
        account = EmailAndPasswordSystemAccount(email=u'johndoe@some.org')
        account.set_new_password(account.email, self.password)
        account.activate()
        return account


class DemoFixture(LoginFixture):
    commit=True
    @set_up
    def do_demo_setup(self):
        self.account


@test(LoginFixture)
def logging_in(fixture):
    """A user can log in by going to the Log in page.
       The name of the currently logged in user is displayed on the home page."""

    browser = fixture.browser
    account = fixture.account

    browser.open(u'/')
    browser.click(XPath.link_with_text(u'Log in'))

    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.type(XPath.input_labelled(u'Password'), u'bobbejaan')
    browser.click(XPath.button_labelled(u'Log in'))

    browser.click(XPath.link_with_text(u'Home'))
    assert browser.is_element_present(XPath.paragraph_containing(u'Welcome johndoe@some.org'))
        

@test(LoginFixture)
def domain_exception(fixture):
    """Typing the wrong password results in an error message being shown to the user."""

    browser = fixture.browser
    account = fixture.account

    browser.open(u'/')
    browser.click(XPath.link_with_text(u'Log in'))

    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.type(XPath.input_labelled(u'Password'), u'wrong password')
    browser.click(XPath.button_labelled(u'Log in'))

    assert browser.is_element_present(XPath.paragraph_containing(u'Invalid login credentials'))






from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, P, HMenu
from reahl.webelixirimpl import WebUserSession
from reahl.domainui.accounts import AccountRegion



class MenuPage(TwoColumnPage):
    def __init__(self, view, main_bookmarks):
        super(MenuPage, self).__init__(view, style=u'basic')
        self.header.add_child(HMenu.from_bookmarks(view, main_bookmarks))


class LoginApp(Region):
    def assemble(self):
        user_session = WebUserSession.for_current_session()
        if user_session.account:
            logged_in_as = user_session.account.email
        else:
            logged_in_as = u'Guest'

        home = self.define_view(u'/', title=u'Home')
        home.set_slot(u'main', P.factory(text=u'Welcome %s' % logged_in_as))

        class LegalBookmarks(object):
            terms_bookmark = home.as_bookmark(self, description=u'Terms of service')
            privacy_bookmark = home.as_bookmark(self, description=u'Privacy policy')
            disclaimer_bookmark = home.as_bookmark(self, description=u'Disclaimer')

        accounts = self.define_region(u'/accounts', AccountRegion,
                                      {u'main_slot': u'main'},
                                      name=u'accounts', bookmarks=LegalBookmarks)

        account_bookmarks = [accounts.get_bookmark(relative_path=relative_path) 
                             for relative_path in [u'/login', u'/register']]
        bookmarks = [home.as_bookmark(self)]+account_bookmarks
        self.define_main_window(MenuPage, bookmarks)






from reahl.tofu import test

from reahl.web_dev.fixtures import WebFixture
from reahl.webdev.tools import XPath

from reahl.doc.examples.tutorial.parameterised2.parameterised2 import AddressBookApp, Address


class AddressAppErrorFixture(WebFixture):
    def new_webapp(self):
        return super(AddressAppErrorFixture, self).new_webapp(site_root=AddressBookApp, enable_js=True)
        
    def new_existing_address(self):
        address = Address(name=u'John Doe', email_address=u'johndoe@some.org')
        address.save()
        return address

    def error_is_displayed(self, text):
        return self.driver_browser.is_element_present(XPath.error_label_containing(text))


@test(AddressAppErrorFixture)
def edit_errors(fixture):
    """Email addresses on the Edit an address page have to be valid email addresses."""
    fixture.reahl_server.set_app(fixture.webapp)
    browser = fixture.driver_browser
    existing_address = fixture.existing_address # Doing this just creates the Address in the database
    
    browser.open(u'/')
    browser.click(XPath.button_labelled(u'Edit'))
    
    browser.type(XPath.input_labelled(u'Email'), u'invalid email address')

    assert fixture.error_is_displayed(u'Email should be a valid email address')









from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, P
from reahl.component.modelinterface import exposed, EmailField, Field, Event, Action, Not

class PageFlowApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')  

        comment = Comment()

        home = self.define_view(u'/', title=u'Page flow demo')
        home.set_slot(u'main', CommentForm.factory(comment))

        thanks = self.define_view(u'/thanks', title=u'Thank you!')
        thanks.set_slot(u'main', P.factory(text=u'Thanks for submitting your comment'))

        none_submitted = self.define_view(u'/none', title=u'Nothing to say?')
        none_submitted.set_slot(u'main', P.factory(text=u'Mmm, you submitted an empty comment??'))

        self.define_transition(comment.events.submit, home, thanks, guard=Action(comment.contains_text))
        self.define_transition(comment.events.submit, home, none_submitted, guard=Not(Action(comment.contains_text)))

        
class Comment(object):
    @exposed
    def fields(self, fields):
        fields.email_address = EmailField(label=u'Email address', required=True)
        fields.text = Field(label=u'Comment')

    @exposed
    def events(self, events):
        events.submit = Event(label=u'Submit', action=Action(self.submit))

    def submit(self):
        print '%s submitted a comment:' % self.email_address
        print self.text

    def contains_text(self):
        return self.text and self.text.strip() != u''


class CommentForm(Form):
    def __init__(self, view, comment):
        super(CommentForm, self).__init__(view, u'myform')

        self.add_child( LabelledBlockInput(TextInput(self, comment.fields.email_address)) )
        self.add_child( LabelledBlockInput(TextInput(self, comment.fields.text)) )
        self.add_child( Button(self, comment.events.submit) )








import datetime

import elixir

from reahl.sqlalchemysupport import Session, metadata

from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, P, H, InputGroup, VMenu
from reahl.component.modelinterface import exposed, EmailField, Field, Event, Action
from reahl.component.i18n import Translator
import babel.dates 


_ = Translator(u'reahl-doc')


class AddressBookPage(TwoColumnPage):
    def __init__(self, view):
        super(AddressBookPage, self).__init__(view, style=u'basic')
        self.secondary.add_child(VMenu.from_languages(view))


class AddressBookApp(Region):
    def assemble(self):
        self.define_main_window(AddressBookPage)
        find = self.define_view(u'/', title=_(u'Address Book'))
        find.set_slot(u'main', AddressBookPanel.factory())



class AddressBookPanel(Panel):
    def __init__(self, view):
        super(AddressBookPanel, self).__init__(view)

        self.add_child(H(view, 1, text=_.ngettext(u'Address', u'Addresses', Address.query.count())))
        
        for address in Address.query.all():
            self.add_child(AddressBox(view, address))

        self.add_child(AddAddressForm(view))



class AddAddressForm(Form):
    def __init__(self, view):
        super(AddAddressForm, self).__init__(view, u'add_form')

        new_address = Address()

        grouped_inputs = self.add_child(InputGroup(view, label_text=_(u'Add an address')))
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_address.fields.name)) )
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_address.fields.email_address)) )

        self.define_event_handler(new_address.events.save)
        grouped_inputs.add_child(Button(self, new_address.events.save))


class AddressBox(Panel):
    def __init__(self, view, address):
        super(AddressBox, self).__init__(view)
        formatted_date = babel.dates.format_date(address.added_date, locale=_.current_locale)
        self.add_child(P(view, text=u'%s: %s (%s)' % (address.name, address.email_address, formatted_date)))


class Address(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    elixir.using_mapper_options(save_on_init=False)
    
    email_address = elixir.Field(elixir.UnicodeText)
    name          = elixir.Field(elixir.UnicodeText)
    added_date    = elixir.Field(elixir.Date)

    @exposed
    def fields(self, fields):
        fields.name = Field(label=_(u'Name'), required=True)
        fields.email_address = EmailField(label=_(u'Email'), required=True)

    def save(self):
        self.added_date = datetime.date.today()
        Session.add(self)
        
    @exposed
    def events(self, events):
        events.save = Event(label=_(u'Save'), action=Action(self.save))






import elixir

from reahl.sqlalchemysupport import Session, metadata

from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, P, H, InputGroup
from reahl.component.modelinterface import exposed, EmailField, Field, Event, Action
from reahl.component.config import Configuration, ConfigSetting
from reahl.component.context import ExecutionContext


class AddressConfig(Configuration):
    filename = u'componentconfig.config.py'
    config_key = u'componentconfig'

    showheader = ConfigSetting(default=False, description=u'Whether the title should be shown as a heading too')


class AddressBookApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')
        find = self.define_view(u'/', title=u'Addresses')
        find.set_slot(u'main', AddressBookPanel.factory())


class AddressBookPanel(Panel):
    def __init__(self, view):
        super(AddressBookPanel, self).__init__(view)

        config = ExecutionContext.get_context().config
        if config.componentconfig.showheader:
            self.add_child(H(view, 1, text=u'Addresses'))
        
        for address in Address.query.all():
            self.add_child(AddressBox(view, address))

        self.add_child(AddAddressForm(view))


class AddAddressForm(Form):
    def __init__(self, view):
        super(AddAddressForm, self).__init__(view, u'add_form')

        new_address = Address()

        grouped_inputs = self.add_child(InputGroup(view, label_text=u'Add an address'))
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_address.fields.name)) )
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_address.fields.email_address)) )

        self.define_event_handler(new_address.events.save)
        grouped_inputs.add_child(Button(self, new_address.events.save))


class AddressBox(Panel):
    def __init__(self, view, address):
        super(AddressBox, self).__init__(view)
        self.add_child(P(view, text=u'%s: %s' % (address.name, address.email_address)))


class Address(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata, tablename=u'tutorial_componentconfig_address')
    elixir.using_mapper_options(save_on_init=False)
    
    email_address = elixir.Field(elixir.UnicodeText)
    name          = elixir.Field(elixir.UnicodeText)

    @exposed
    def fields(self, fields):
        fields.name = Field(label=u'Name', required=True)
        fields.email_address = EmailField(label=u'Email', required=True)

    def save(self):
        Session.add(self)
        
    @exposed
    def events(self, events):
        events.save = Event(label=u'Save', action=Action(self.save))




import hashlib

import elixir

from reahl.sqlalchemysupport import Session, metadata
from reahl.elixirsupport import session_scoped

from reahl.component.exceptions import DomainException
from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, P, H, InputGroup, HMenu,\
                         PasswordInput, ErrorFeedbackMessage
from reahl.component.modelinterface import exposed, EmailField, Field, Event, Action, PasswordField



class User(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    
    email_address = elixir.Field(elixir.UnicodeText, required=True)
    name          = elixir.Field(elixir.UnicodeText, required=True)
    password_md5  = elixir.Field(elixir.String, required=True)

    def set_password(self, password):
        self.password_md5 = hashlib.md5(password).hexdigest()
        
    def matches_password(self, password):
        return self.password_md5 == hashlib.md5(password).hexdigest()


@session_scoped
class LoginSession(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata, tablename=u'tutorial_loginsession')

    current_user = elixir.ManyToOne(User)
    email_address = elixir.Field(elixir.UnicodeText)

    @exposed
    def fields(self, fields):
        fields.email_address = EmailField(label=u'Email', required=True)
        fields.password = PasswordField(label=u'Password', required=True)

    @exposed
    def events(self, events):
        events.log_in = Event(label=u'Log in', action=Action(self.log_in))

    def log_in(self):
        user = User.query.filter_by(email_address=self.email_address).one()
        if user.matches_password(self.password):
            self.current_user = user
        else:
            raise InvalidPassword()



class MenuPage(TwoColumnPage):
    def __init__(self, view, main_bookmarks):
        super(MenuPage, self).__init__(view, style=u'basic')
        self.header.add_child(HMenu.from_bookmarks(view, main_bookmarks))


class InvalidPassword(DomainException):
    def as_user_message(self):
        return u'The email/password given do not match.'
    


class LoginForm(Form):
    def __init__(self, view, login_session):
        super(LoginForm, self).__init__(view, u'login')
        
        if self.exception:
            self.add_child(ErrorFeedbackMessage(view, self.exception.as_user_message()))

        self.add_child(LabelledBlockInput(TextInput(self, login_session.fields.email_address)))
        self.add_child(LabelledBlockInput(PasswordInput(self, login_session.fields.password)))

        self.define_event_handler(login_session.events.log_in)
        self.add_child(Button(self, login_session.events.log_in))


class SessionScopeApp(Region):
    def assemble(self):
        login_session = LoginSession.for_current_session()
        if login_session.current_user:
            user_name = login_session.current_user.name
        else:
            user_name = u'Guest'

        home = self.define_view(u'/', title=u'Home')
        home.set_slot(u'main', P.factory(text=u'Welcome %s' % user_name))

       
        login_page = self.define_view(u'/login', title=u'Log in')
        login_page.set_slot(u'main', LoginForm.factory(login_session))
        
        bookmarks = [i.as_bookmark(self) for i in [home, login_page]]
        self.define_main_window(MenuPage, bookmarks)







from reahl.tofu import test, set_up

from reahl.web_dev.fixtures import WebFixture
from reahl.webdev.tools import Browser, XPath

from reahl.doc.examples.tutorial.sessionscope.sessionscope import SessionScopeApp, User


class SessionScopeFixture(WebFixture):
    def new_browser(self):
        return Browser(self.new_webapp(site_root=SessionScopeApp))
        
    password = u'bobbejaan'

    def new_user(self):
        user = User(name=u'John Doe', email_address=u'johndoe@some.org')
        user.set_password(self.password)
        return user


class DemoFixture(SessionScopeFixture):
    commit=True
    @set_up
    def make_stuff(self):
        self.user


@test(SessionScopeFixture)
def logging_in(fixture):
    """A user can log in by going to the Log in page.
       The name of the currently logged in user is displayed on the home page."""

    browser = fixture.browser
    user = fixture.user

    browser.open(u'/')
    browser.click(XPath.link_with_text(u'Log in'))

    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.type(XPath.input_labelled(u'Password'), u'bobbejaan')
    browser.click(XPath.button_labelled(u'Log in'))

    browser.click(XPath.link_with_text(u'Home'))
    assert browser.is_element_present(XPath.paragraph_containing(u'Welcome John Doe'))
    
@test(SessionScopeFixture)
def email_retained(fixture):
    """The email address used when last logged in is always pre-populated on the Log in page."""

    browser = fixture.browser
    user = fixture.user

    browser.open(u'/')
    browser.click(XPath.link_with_text(u'Log in'))

    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.type(XPath.input_labelled(u'Password'), u'bobbejaan')
    browser.click(XPath.button_labelled(u'Log in'))

    # Go away from the page, then back
    browser.click(XPath.link_with_text(u'Home'))
    browser.click(XPath.link_with_text(u'Log in'))

    # .. then the email is still pre-populated
    typed_value = browser.get_value(XPath.input_labelled(u'Email'))
    assert typed_value == u'johndoe@some.org'
    

@test(SessionScopeFixture)
def domain_exception(fixture):
    """Typing the wrong password results in an error message being shown to the user."""

    browser = fixture.browser
    user = fixture.user

    browser.open(u'/')
    browser.click(XPath.link_with_text(u'Log in'))

    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.type(XPath.input_labelled(u'Password'), u'wrong password')
    browser.click(XPath.button_labelled(u'Log in'))

    assert browser.is_element_present(XPath.paragraph_containing(u'The email/password given do not match'))





import elixir
from sqlalchemy.orm.exc import NoResultFound

from reahl.sqlalchemysupport import Session, metadata

from reahl.web.fw import Region, Url, UrlBoundView, CannotCreate
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, P, H, A, InputGroup, HMenu
from reahl.component.modelinterface import exposed, EmailField, Field, Event, IntegerField, Action


class AddressBookPage(TwoColumnPage):
    def __init__(self, view, main_bookmarks):
        super(AddressBookPage, self).__init__(view, style=u'basic')
        self.header.add_child(HMenu.from_bookmarks(view, main_bookmarks))


class EditView(UrlBoundView):
    def assemble(self, address_id=None):
        try:
            address = Address.query.filter_by(id=address_id).one()
        except NoResultFound:
            raise CannotCreate()

        self.title = u'Edit %s' % address.name
        self.set_slot(u'main', EditAddressForm.factory(address))


class AddressBookApp(Region):
    def assemble(self):

        add = self.define_view(u'/add', title=u'Add an address')
        add.set_slot(u'main', AddAddressForm.factory())

        edit = self.define_view(u'/edit', view_class=EditView, address_id=IntegerField())

        addresses = self.define_view(u'/', title=u'Addresses')
        addresses.set_slot(u'main', AddressBookPanel.factory(self))

        bookmarks = [f.as_bookmark(self) for f in [addresses, add]]
        self.define_main_window(AddressBookPage, bookmarks)

        self.define_transition(Address.events.save, add, addresses)
        self.define_transition(Address.events.update, edit, addresses)

        self.edit = edit

    def get_edit_bookmark(self, address, description=None):
        return self.edit.as_bookmark(self, address_id=address.id, description=description)



class AddressBookPanel(Panel):
    def __init__(self, view, address_book_app):
        super(AddressBookPanel, self).__init__(view)

        self.add_child(H(view, 1, text=u'Addresses'))
        
        for address in Address.query.all():
            self.add_child(AddressBox(view, address, address_book_app))


class EditAddressForm(Form):
    def __init__(self, view, address):
        super(EditAddressForm, self).__init__(view, u'edit_form')

        grouped_inputs = self.add_child(InputGroup(view, label_text=u'Edit address'))
        grouped_inputs.add_child(LabelledBlockInput(TextInput(self, address.fields.name)))
        grouped_inputs.add_child(LabelledBlockInput(TextInput(self, address.fields.email_address)))

        grouped_inputs.add_child(Button(self, address.events.update))


class AddAddressForm(Form):
    def __init__(self, view):
        super(AddAddressForm, self).__init__(view, u'add_form')

        new_address = Address()
        grouped_inputs = self.add_child(InputGroup(view, label_text=u'Add an address'))
        grouped_inputs.add_child(LabelledBlockInput(TextInput(self, new_address.fields.name)))
        grouped_inputs.add_child(LabelledBlockInput(TextInput(self, new_address.fields.email_address)))

        grouped_inputs.add_child(Button(self, new_address.events.save))


class AddressBox(Panel):
    def __init__(self, view, address, address_book_app):
        super(AddressBox, self).__init__(view)

        bookmark = address_book_app.get_edit_bookmark(address=address, description=u'edit')
        par = self.add_child(P(view, text=u'%s: %s ' % (address.name, address.email_address)))
        par.add_child(A.from_bookmark(view, bookmark))


class Address(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata, tablename=u'tutorial_parameterised1_address')
    elixir.using_mapper_options(save_on_init=False)
    
    email_address = elixir.Field(elixir.UnicodeText)
    name          = elixir.Field(elixir.UnicodeText)

    @exposed
    def fields(self, fields):
        fields.name = Field(label=u'Name', required=True)
        fields.email_address = EmailField(label=u'Email', required=True)

    @exposed('save', 'update')
    def events(self, events):
        events.save = Event(label=u'Save', action=Action(self.save))
        events.update = Event(label=u'Update')

    def save(self):
        Session.add(self)







from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, P, HMenu


class MyCustomPage(TwoColumnPage):
    def __init__(self, view, bookmarks):
        super(MyCustomPage, self).__init__(view, style=u'basic')
        self.header.add_child(HMenu.from_bookmarks(view, bookmarks))

class SlotsApp(Region):
    def assemble(self):

        home = self.define_view(u'/', title=u'Page 1')
        home.set_slot(u'main', P.factory(text=u'In this slot will be some main content for the view on /'))
        home.set_slot(u'secondary', P.factory(text=u'Some secondary content related to /'))

        another = self.define_view(u'/page2', title=u'Page 2')
        another.set_slot(u'main', P.factory(text=u'This could, for example, be where a photo gallery shows a large photo.'))
        another.set_slot(u'secondary', P.factory(text=u'Thumbnails will then sit on the side of the big photo.'))

        bookmarks = [home.as_bookmark(self), another.as_bookmark(self)]
        self.define_main_window(MyCustomPage, bookmarks)






from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, P, H, InputGroup, HMenu,\
                         PasswordInput, ErrorFeedbackMessage
from reahl.systemaccountmodel import AccountManagementInterface, UserSession



class MenuPage(TwoColumnPage):
    def __init__(self, view, main_bookmarks):
        super(MenuPage, self).__init__(view, style=u'basic')
        self.header.add_child(HMenu.from_bookmarks(view, main_bookmarks))


class LoginForm(Form):
    def __init__(self, view):
        super(LoginForm, self).__init__(view, u'login')
        
        accounts = AccountManagementInterface.for_current_session()

        if self.exception:
            self.add_child(ErrorFeedbackMessage(view, self.exception.as_user_message()))

        self.add_child(LabelledBlockInput(TextInput(self, accounts.fields.email)))
        self.add_child(LabelledBlockInput(PasswordInput(self, accounts.fields.password)))

        self.define_event_handler(accounts.events.login_event)
        self.add_child(Button(self, accounts.events.login_event))


class LoginApp(Region):
    def assemble(self):
        user_session = UserSession.for_current_session()
        if user_session.account:
            logged_in_as = user_session.account.email
        else:
            logged_in_as = u'Guest'

        home = self.define_view(u'/', title=u'Home')
        home.set_slot(u'main', P.factory(text=u'Welcome %s' % logged_in_as))

        login_page = self.define_view(u'/login', title=u'Log in')
        login_page.set_slot(u'main', LoginForm.factory())
        
        bookmarks = [i.as_bookmark(self) for i in [home, login_page]]
        self.define_main_window(MenuPage, bookmarks)







from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, Panel, P, H, InputGroup

class AddressBookApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')
        find = self.define_view(u'/', title=u'Addresses')
        find.set_slot(u'main', AddressBookPanel.factory())


class AddressBookPanel(Panel):
    def __init__(self, view):
        super(AddressBookPanel, self).__init__(view)

        self.add_child(H(view, 1, text=u'Addresses'))

        for address in Address.query.all():
            self.add_child(AddressBox(view, address))

        self.add_child(AddAddressForm(view))


class AddAddressForm(Form):
    def __init__(self, view):
        super(AddAddressForm, self).__init__(view, u'add_form')
        self.add_child(InputGroup(view, label_text=u'Add an address'))


class AddressBox(Panel):
    def __init__(self, view, address):
        super(AddressBox, self).__init__(view)
        self.add_child(P(view, text=u'%s: %s' % (address.name, address.email_address)))


# The model from before:
import elixir
from reahl.sqlalchemysupport import Session, metadata
from reahl.component.modelinterface import Field, Event, EmailField, Action, exposed

class Address(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    elixir.using_mapper_options(save_on_init=False)

    email_address = elixir.Field(elixir.UnicodeText)
    name          = elixir.Field(elixir.UnicodeText)

    def save(self):
        Session.add(self)

    @exposed
    def fields(self, fields):
        fields.name = Field(label=u'Name', required=True)
        fields.email_address = EmailField(label=u'Email', required=True)

    @exposed
    def events(self, events):
        events.save = Event(label=u'Save', action=Action(self.save))






import elixir

from reahl.sqlalchemysupport import Session, metadata

from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, P, H, InputGroup
from reahl.component.modelinterface import exposed, EmailField, Field, Event, Action


class PersistenceApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')
        home = self.define_view(u'/', title=u'Persistence demo')
        home.set_slot(u'main', CommentPostPanel.factory())


class Comment(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata, tablename=u'features_comment')
    elixir.using_mapper_options(save_on_init=False)
    
    email_address = elixir.Field(elixir.UnicodeText)
    text          = elixir.Field(elixir.UnicodeText)
    
    @exposed
    def fields(self, fields):
        fields.email_address = EmailField(label=u'Email address', required=True)
        fields.text          = Field(label=u'Comment', required=True)

    @exposed
    def events(self, events):
        events.submit = Event(label=u'Submit', action=Action(self.submit))

    def submit(self):
        Session.add(self)


class CommentPostPanel(Panel):
    def __init__(self, view):
        super(CommentPostPanel, self).__init__(view)

        self.add_child(CommentForm(view))

        for comment in Comment.query.all():
            self.add_child(CommentBox(view, comment))


class CommentForm(Form):
    def __init__(self, view):
        super(CommentForm, self).__init__(view, u'myform')

        new_comment = Comment()
        grouped_inputs = self.add_child(InputGroup(view, label_text=u'Leave a comment'))
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_comment.fields.email_address)) )
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_comment.fields.text)) )
        self.define_event_handler(new_comment.events.submit)
        grouped_inputs.add_child( Button(self, new_comment.events.submit) )


class CommentBox(Panel):
    def __init__(self, view, comment):
        super(CommentBox, self).__init__(view)
        self.add_child(P(view, text=u'By %s: %s' % (comment.email_address, comment.text)))
        





from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput
from reahl.component.modelinterface import exposed, EmailField

class ValidationApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')  

        home = self.define_view(u'/', title=u'Validation demo')
        home.set_slot(u'main', CommentForm.factory())


class Comment(object):
    @exposed
    def fields(self, fields):
        fields.email_address = EmailField(label=u'Email address', required=True)


class CommentForm(Form):
    def __init__(self, view):
        super(CommentForm, self).__init__(view, u'myform')

        comment = Comment()
        self.add_child( TextInput(self, comment.fields.email_address) )








import elixir
from sqlalchemy.orm.exc import NoResultFound

from reahl.sqlalchemysupport import Session, metadata
from reahl.systemaccountmodel import EmailAndPasswordSystemAccount


class Address(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    elixir.using_mapper_options(save_on_init=False)
    
    address_book  = elixir.ManyToOne(u'reahl.doc.examples.tutorial.access1.access1.AddressBook')
    email_address = elixir.Field(elixir.UnicodeText)
    name          = elixir.Field(elixir.UnicodeText)

    def save(self):
        Session.add(self)

        


class AddressBook(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)

    owner      = elixir.ManyToOne(EmailAndPasswordSystemAccount, required=True)

    @classmethod
    def owned_by(cls, account):
        return cls.query.filter_by(owner=account)
        
    @classmethod
    def address_books_visible_to(cls, account):
        visible_books = cls.query.join(Collaborator).filter(Collaborator.account == account).all()
        visible_books.extend(cls.owned_by(account))
        return visible_books

    # See https://groups.google.com/forum/?fromgroups=#!topic/sqlelixir/ZlR9Kvcor6Q
    #    addresses  = elixir.OneToMany(Address)
    @property
    def addresses(self):
        return Address.query.filter_by(address_book=self).all()

    collaborators = elixir.OneToMany(u'reahl.doc.examples.tutorial.access1.access1.Collaborator', lazy=u'dynamic')

    def allow(self, account, can_add_addresses=False, can_edit_addresses=False):
        Collaborator.query.filter_by(address_book=self, account=account).delete()
        Collaborator(address_book=self, account=account,
            can_add_addresses=can_add_addresses,
            can_edit_addresses=can_edit_addresses)

    def can_be_edited_by(self, account):
        if account is self.owner: 
            return True
            
        collaborator = self.get_collaborator(account)
        return (collaborator and collaborator.can_edit_addresses) or self.can_be_added_to_by(account)

    def can_be_added_to_by(self, account):
        if account is self.owner: 
            return True

        collaborator = self.get_collaborator(account)
        return collaborator and collaborator.can_add_addresses
        
    def collaborators_can_be_added_by(self, account):
        return self.owner is account

    def is_visible_to(self, account):
        return self in self.address_books_visible_to(account)

    def get_collaborator(self, account):            
        collaborators = self.collaborators.filter_by(account=account)
        count = collaborators.count()
        if count == 1:
            return collaborators.one()
        if count > 1:
            raise ProgrammerError('There can be only one Collaborator per account. Here is more than one.')
        return None


class Collaborator(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)

    account = elixir.ManyToOne(EmailAndPasswordSystemAccount)
    can_add_addresses = elixir.Field(elixir.Boolean, default=False)
    can_edit_addresses = elixir.Field(elixir.Boolean, default=False)

    address_book = elixir.ManyToOne(AddressBook)






from helloapache import HelloApp

# The Region class acting as the root of the URL hierachy of your application:
web.site_root = HelloApp       

# HTTP is served on port 80, and HTTPS on 443, except when in development!
web.default_http_port = 80
web.encrypted_http_port = 443

# Each application has one (and only one) directory where static files can be served from
# (This directory needs to be owned by www-data)
web.static_root = '/var/local/helloapache/www'  




from setuptools import setup
setup(
    name='reahl-doc',
    version=u'2.0.0',
    description='Documentation and examples for Reahl.',
    long_description=u'Reahl is a web application framework that allows a Python programmer to work in terms of useful abstractions - using a single programming language.\n\nReahl-doc contains documentation and examples of Reahl.\n\nSee http://www.reahl.org/docs/current/tutorial/gettingstarted.d.html for installation instructions. ',
    url=u'http://www.reahl.org',
    maintainer='Iwan Vosloo',
    maintainer_email='iwan@reahl.org',
    packages=['reahl', 'reahl.doc', 'reahl.doc.examples', 'reahl.doc.examples.features', 'reahl.doc.examples.tutorial', 'reahl.doc.examples.features.i18nexample', 'reahl.doc.examples.features.pageflow', 'reahl.doc.examples.features.persistence', 'reahl.doc.examples.features.validation', 'reahl.doc.examples.features.tabbedpanel', 'reahl.doc.examples.features.access', 'reahl.doc.examples.features.slots', 'reahl.doc.examples.features.layout', 'reahl.doc.examples.tutorial.login2', 'reahl.doc.examples.tutorial.pageflow1', 'reahl.doc.examples.tutorial.i18nexample', 'reahl.doc.examples.tutorial.hello', 'reahl.doc.examples.tutorial.parameterised1', 'reahl.doc.examples.tutorial.migration', 'reahl.doc.examples.tutorial.parameterised2', 'reahl.doc.examples.tutorial.ajax', 'reahl.doc.examples.tutorial.sessionscope', 'reahl.doc.examples.tutorial.componentconfig', 'reahl.doc.examples.tutorial.login1', 'reahl.doc.examples.tutorial.pageflow2', 'reahl.doc.examples.tutorial.pager', 'reahl.doc.examples.tutorial.addressbook2', 'reahl.doc.examples.tutorial.helloapache', 'reahl.doc.examples.tutorial.addressbook1', 'reahl.doc.examples.tutorial.access1', 'reahl.doc.examples.tutorial.jobs', 'reahl.doc.examples.tutorial.access', 'reahl.doc.examples.tutorial.access2', 'reahl.doc.examples.tutorial.login2.login2_dev', 'reahl.doc.examples.tutorial.i18nexample.i18nexamplemessages', 'reahl.doc.examples.tutorial.i18nexample.i18nexample_dev', 'reahl.doc.examples.tutorial.migration.migration_dev', 'reahl.doc.examples.tutorial.parameterised2.parameterised2_dev', 'reahl.doc.examples.tutorial.ajax.ajax_dev', 'reahl.doc.examples.tutorial.sessionscope.sessionscope_dev', 'reahl.doc.examples.tutorial.componentconfig.componentconfig_dev', 'reahl.doc.examples.tutorial.login1.login1_dev', 'reahl.doc.examples.tutorial.pager.pager_dev', 'reahl.doc.examples.tutorial.access1.access1_dev', 'reahl.doc.examples.tutorial.jobs.jobs_dev', 'reahl.doc.examples.tutorial.access.access_dev', 'reahl.doc.examples.tutorial.access2.access2_dev'],
    py_modules=['setup'],
    include_package_data=True,
    package_data={'': [u'*/LC_MESSAGES/*.mo']},
    namespace_packages=['reahl'],
    install_requires=[u'reahl-web>=2.0,<2.1', u'reahl-component>=2.0,<2.1', u'reahl-sqlalchemysupport>=2.0,<2.1', u'reahl-web-elixirimpl>=2.0,<2.1', u'reahl-domain>=2.0,<2.1', u'reahl-domainui>=2.0,<2.1'],
    setup_requires=[],
    tests_require=[u'reahl-tofu>=2.0,<2.1', u'Sphinx', u'reahl-stubble>=2.0,<2.1', u'reahl-dev>=2.0,<2.1', u'reahl-webdev>=2.0,<2.1'],
    test_suite=u'reahl.doc_dev',
    entry_points={
        'reahl.translations': [
            u'reahl-doc = reahl.doc.examples.tutorial.i18nexample.i18nexamplemessages'    ],
        u'reahl.configspec': [
            u'config = reahl.doc.examples.tutorial.componentconfig.componentconfig:AddressConfig'    ],
        u'reahl.dev.commands': [
            u'GetExample = reahl.doc.commands:GetExample',
            u'ListExamples = reahl.doc.commands:ListExamples'    ],
        'reahl.eggs': [
            u'Egg = reahl.component.eggs:ReahlEgg'    ],
        u'reahl.scheduled_jobs': [
            u'Address.clear_added_flags = reahl.doc.examples.tutorial.jobs.jobs:Address.clear_added_flags'    ],
        u'reahl.persistlist': [
            u'0 = reahl.doc.examples.features.persistence.persistence:Comment',
            u'1 = reahl.doc.fileupload:Comment',
            u'2 = reahl.doc.fileupload:AttachedFile',
            u'3 = reahl.doc.examples.tutorial.addressbook2.addressbook2:Address',
            u'4 = reahl.doc.examples.tutorial.addressbook1.addressbook1:Address',
            u'5 = reahl.doc.examples.tutorial.pageflow2.pageflow2:Address',
            u'6 = reahl.doc.examples.tutorial.pageflow1.pageflow1:Address',
            u'7 = reahl.doc.examples.tutorial.parameterised1.parameterised1:Address',
            u'8 = reahl.doc.examples.tutorial.parameterised2.parameterised2:Address',
            u'9 = reahl.doc.examples.tutorial.sessionscope.sessionscope:User',
            u'10 = reahl.doc.examples.tutorial.sessionscope.sessionscope:LoginSession',
            u'11 = reahl.doc.examples.tutorial.access1.access1:AddressBook',
            u'12 = reahl.doc.examples.tutorial.access1.access1:Collaborator',
            u'13 = reahl.doc.examples.tutorial.access1.access1:Address',
            u'14 = reahl.doc.examples.tutorial.access2.access2:AddressBook',
            u'15 = reahl.doc.examples.tutorial.access2.access2:Collaborator',
            u'16 = reahl.doc.examples.tutorial.access2.access2:Address',
            u'17 = reahl.doc.examples.tutorial.access.access:AddressBook',
            u'18 = reahl.doc.examples.tutorial.access.access:Collaborator',
            u'19 = reahl.doc.examples.tutorial.access.access:Address',
            u'20 = reahl.doc.examples.tutorial.i18nexample.i18nexample:Address',
            u'21 = reahl.doc.examples.tutorial.componentconfig.componentconfig:Address',
            u'22 = reahl.doc.examples.tutorial.migration.migration:Address',
            u'23 = reahl.doc.examples.tutorial.jobs.jobs:Address'    ],
                 },
    extras_require={u'pillow': [u'pillow']}
)



# Copyright 2012, 2013 Reahl Software Services (Pty) Ltd. All rights reserved.
#
#    This file is part of Reahl.
#
#    Reahl is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation; version 3 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    
import os
import os.path
import subprocess

from nose.tools import istest
from reahl.tofu import test, set_up, tear_down, scenario, Fixture
from reahl.tofu import vassert, expected, temp_file_with
from reahl.stubble import SystemOutStub
from reahl.component.config import StoredConfiguration

from reahl.web_dev.fixtures import WebBasicsMixin
from reahl.webdev.fixtures import BrowserSetup
from reahl.webdev.tools import XPath, Browser

from reahl.doc.examples.tutorial.hello.hello import HelloApp
from reahl.doc.examples.tutorial.helloapache import helloapache
from reahl.doc.examples.features.tabbedpanel.tabbedpanel import TabbedPanelApp
from reahl.doc.examples.features.validation.validation import ValidationApp
from reahl.doc.examples.features.layout.layout import LayoutApp
from reahl.doc.examples.features.pageflow.pageflow import PageFlowApp
from reahl.doc.examples.features.persistence.persistence import PersistenceApp
from reahl.doc.examples.features.access.access import AccessApp
from reahl.doc.examples.features.i18nexample.i18nexample import TranslatedApp

from reahl.doc.examples.features.slots.slots import SlotsApp

from reahl.doc.fileupload import FileUploadApp, AttachedFile

from reahl.doc.basichtmlwidgets import BasicHTMLWidgetsApp
from reahl.doc.basichtmlinputs import BasicHTMLInputsApp

from reahl.doc.examples.tutorial.addressbook1 import addressbook1
from reahl.doc.examples.tutorial.addressbook2 import addressbook2
from reahl.doc.examples.tutorial.pageflow1 import pageflow1
from reahl.doc.examples.tutorial.pageflow2 import pageflow2
from reahl.doc.examples.tutorial.parameterised1 import parameterised1
from reahl.doc.examples.tutorial.parameterised2 import parameterised2


class ExampleFixture(Fixture, WebBasicsMixin):
    def start_example_app(self):
        self.reahl_server.set_app(self.webapp)

    def new_screenshot_directory(self):
        relative_path = u'doc/_build/screenshots'
        if not os.path.isdir(relative_path):
            os.makedirs(relative_path)
        return relative_path

    def new_screenshot_path(self, filename=u'screenshot.png'):
        return os.path.join(os.getcwd(), self.screenshot_directory, filename)
    
    def tab_is_active(self, tab_name):
        return self.driver_browser.execute_script('return window.jQuery("a:contains(\'%s\')").parent().hasClass("active")' % tab_name)

    def tab_contents_equals(self, expected_contents):
        return self.driver_browser.execute_script('return window.jQuery("div.reahl-tabbedpanel div p").html() == "%s"' % expected_contents)

    def error_is_visible(self):
        return self.driver_browser.execute_script('return window.jQuery(".reahl-form label.error").is(":visible")')

    def is_error_text(self, text):
        return text == self.driver_browser.get_text(u"//form/label[@class='error']")

    def get_main_slot_contents(self):
        return self.driver_browser.get_text('//div[@id="yui-main"]//p')

    def get_secondary_slot_contents(self):
        return self.driver_browser.get_text('//div[@id="yui-main"]/following-sibling::div/p')

    def uploaded_file_is_listed(self, filename):
        return self.driver_browser.is_element_present('//ul/li/span[text()="%s"]' % os.path.basename(filename))

    @scenario
    def hello(self):
        self.webapp = self.new_webapp(site_root=HelloApp)

    @scenario
    def hello_apache(self):
        self.webapp = self.new_webapp(site_root=helloapache.HelloApp)

    @scenario
    def tabbed_panel(self):
        self.webapp = self.new_webapp(site_root=TabbedPanelApp, enable_js=True)

    @scenario
    def validation(self):
        self.webapp = self.new_webapp(site_root=ValidationApp, enable_js=True)

    @scenario
    def layout(self):
        self.webapp = self.new_webapp(site_root=LayoutApp, enable_js=True)

    @scenario
    def pageflow(self):
        self.webapp = self.new_webapp(site_root=PageFlowApp, enable_js=True)

    @scenario
    def persistence(self):
        self.webapp = self.new_webapp(site_root=PersistenceApp, enable_js=True)

    @scenario
    def access_control(self):
        self.webapp = self.new_webapp(site_root=AccessApp, enable_js=True)

    @scenario
    def i18n(self):
        self.webapp = self.new_webapp(site_root=TranslatedApp, enable_js=True)

    @scenario
    def basichtmlinputs(self):
        self.webapp = self.new_webapp(site_root=BasicHTMLInputsApp, enable_js=True)

    @scenario
    def basichtmlwidgets(self):
        self.webapp = self.new_webapp(site_root=BasicHTMLWidgetsApp, enable_js=True)

    @scenario
    def fileupload(self):
        self.webapp = self.new_webapp(site_root=FileUploadApp, enable_js=True)
#        self.webapp = self.new_webapp(site_root=FileUploadApp, enable_js=False)

    @scenario
    def slots(self):
        self.webapp = self.new_webapp(site_root=SlotsApp, enable_js=True)

    @scenario
    def addressbook1(self):
        self.webapp = self.new_webapp(site_root=addressbook1.AddressBookApp)

    @scenario
    def addressbook2(self):
        self.webapp = self.new_webapp(site_root=addressbook2.AddressBookApp)

    @scenario
    def pageflow1(self):
        self.webapp = self.new_webapp(site_root=pageflow1.AddressBookApp)

    @scenario
    def pageflow2(self):
        self.webapp = self.new_webapp(site_root=pageflow2.AddressBookApp)

    @scenario
    def parameterised1(self):
        self.webapp = self.new_webapp(site_root=parameterised1.AddressBookApp)

    @scenario
    def parameterised2(self):
        self.webapp = self.new_webapp(site_root=parameterised2.AddressBookApp)


@test(ExampleFixture)
def hit_home_page(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open(u'/')

@test(ExampleFixture.tabbed_panel)
def widgets(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open(u'/')
    vassert( fixture.driver_browser.wait_for(fixture.tab_is_active, u'Tab 1') )
    vassert( fixture.driver_browser.wait_for(fixture.tab_contents_equals, u'A paragraph to give content to the first tab.') )
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'tabbedpanel1.png'))

    fixture.driver_browser.click(XPath.link_with_text(u'Tab 2'))
    vassert( fixture.driver_browser.wait_for(fixture.tab_is_active, u'Tab 2') )
    vassert( fixture.driver_browser.wait_for(fixture.tab_contents_equals, u'And another ...  to give content to the second tab.') )
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'tabbedpanel2.png'))

@test(ExampleFixture.validation)
def validation(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open(u'/')
    vassert( fixture.driver_browser.wait_for_not(fixture.error_is_visible) )
    vassert( fixture.driver_browser.is_element_present(u'//input') )
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'validation1.png'))
    
    fixture.driver_browser.type(u'//input', u'johndoe')
    fixture.driver_browser.press_tab(u'//input')
    vassert( fixture.driver_browser.wait_for(fixture.error_is_visible) )
    vassert( fixture.is_error_text(u'Email address should be a valid email address') )
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'validation2.png'))

    fixture.driver_browser.type(u'//input', u'')
    fixture.driver_browser.press_tab(u'//input')
    vassert( fixture.driver_browser.wait_for(fixture.error_is_visible) )
    vassert( fixture.is_error_text(u'Email address is required') )
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'validation3.png'))

    fixture.driver_browser.type(u'//input', u'johndoe@some.org')
    fixture.driver_browser.press_tab(u'//input')
    vassert( fixture.driver_browser.wait_for_not(fixture.error_is_visible) )
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'validation4.png'))

@test(ExampleFixture.layout)
def layout(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open(u'/')
    fixture.driver_browser.type(XPath.input_labelled(u'Email address'), u'johndoe')
    fixture.driver_browser.press_tab(u'//input')
    vassert( fixture.driver_browser.wait_for(fixture.error_is_visible) )
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'layout.png'))

@test(ExampleFixture.pageflow)
def pageflow(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open(u'/')
    fixture.driver_browser.type(XPath.input_labelled(u'Email address'), u'johndoe@some.org')
    fixture.driver_browser.type(XPath.input_labelled(u'Comment'), u'')
    with SystemOutStub() as output:
        fixture.driver_browser.click(XPath.button_labelled(u'Submit'))
        
        vassert( output.captured_output == u'johndoe@some.org submitted a comment:\nNone\n' )
        vassert( fixture.driver_browser.current_url.path == u'/none' )
        output.capture_console_screenshot(fixture.new_screenshot_path(u'pageflow1.txt'))

        fixture.driver_browser.open(u'/')
        fixture.driver_browser.type(XPath.input_labelled(u'Email address'), u'johndoe@some.org')
        fixture.driver_browser.type(XPath.input_labelled(u'Comment'), u'some comment text')
        with SystemOutStub() as output:
            fixture.driver_browser.click(XPath.button_labelled(u'Submit'))

        vassert( output.captured_output == u'johndoe@some.org submitted a comment:\nsome comment text\n' )
        vassert( fixture.driver_browser.current_url.path == u'/thanks' )
        output.capture_console_screenshot(fixture.new_screenshot_path(u'pageflow2.txt'))


@test(ExampleFixture.persistence)
def persistence(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open('/')

    vassert( not fixture.driver_browser.is_element_present('//h1') )
    vassert( fixture.driver_browser.is_element_present('//form') )

    fixture.driver_browser.type(XPath.input_labelled(u'Email address'), u'johndoe@some.org')
    fixture.driver_browser.type(XPath.input_labelled(u'Comment'), u'some comment text')
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'persistence1.png'))

    fixture.driver_browser.click(XPath.button_labelled(u'Submit'))

    vassert( fixture.driver_browser.is_element_present('//form') )
    vassert( fixture.driver_browser.is_element_present('//form/following-sibling::div/p') )
    comment_text = fixture.driver_browser.get_text('//form/following-sibling::div/p')
    vassert( comment_text == u'By johndoe@some.org: some comment text' )

    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'persistence2.png'))

@test(ExampleFixture.access_control)
def access(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open('/')
    vassert( fixture.driver_browser.is_element_present(XPath.input_labelled('Greyed out') ))
    vassert( not fixture.driver_browser.is_editable(XPath.input_labelled('Greyed out')) )
    vassert( fixture.driver_browser.is_element_present(XPath.button_labelled('Greyed out button')) )
    vassert( not fixture.driver_browser.is_editable(XPath.button_labelled('Greyed out button')) )
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'access.png'))

    from reahl.doc.examples.features.access.access import Comment
    from reahl.component.exceptions import AccessRestricted
    comment = Comment()
    with expected(AccessRestricted):
        comment.do_something()

@test(ExampleFixture.i18n)
def i18n(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open('/some_page')
    vassert( fixture.get_main_slot_contents() == u'This is a translated string. The current URL is "/some_page".' )
    vassert( fixture.driver_browser.title == u'Translated example' )
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'i18n1.png'))

    fixture.driver_browser.click(XPath.link_with_text(u'Afrikaans'))
    vassert( fixture.get_main_slot_contents() == u'Hierdie is \'n vertaalde string. Die huidige URL is "/af/some_page".' )
    vassert( fixture.driver_browser.title == u'Vertaalde voorbeeld' )
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'i18n2.png'))

@test(ExampleFixture.basichtmlwidgets)
def basichtmlwidgets(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open('/')

@test(ExampleFixture.fileupload)
def fileupload(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open('/')

    file1 = temp_file_with(u'some content in a file', u'file1.txt')
    file2 = temp_file_with(u'different content', u'file2.txt')
    file3 = temp_file_with(u'even more content', u'file3.txt')

    # Upload a file
    fixture.driver_browser.type(u'//input[@type="file"]', file1.name)
    vassert( fixture.uploaded_file_is_listed(file1.name) )

    # Upload a file
    fixture.driver_browser.type(u'//input[@type="file"]', file2.name)
    vassert( fixture.uploaded_file_is_listed(file2.name) )

    # Upload a file
    fixture.driver_browser.type(u'//input[@type="file"]', file3.name)
    vassert( fixture.uploaded_file_is_listed(file3.name) )

    # Delete file2 from uploaded files
    fixture.driver_browser.click(XPath.button_labelled(u'Remove', filename=os.path.basename(file2.name)))
    vassert( not fixture.uploaded_file_is_listed(file2.name) )

    # Submit the form
    fixture.driver_browser.type(XPath.input_labelled(u'Email address'), u'johndoe@some.org')
    fixture.driver_browser.type(XPath.input_labelled(u'Comment'), u'some comment text')
    fixture.driver_browser.click(XPath.button_labelled(u'Submit'))

    attached_file1 = AttachedFile.query.filter_by(filename=os.path.basename(file1.name)).one()
    attached_file3 = AttachedFile.query.filter_by(filename=os.path.basename(file3.name)).one()
    vassert( AttachedFile.query.count() == 2 )
    vassert( attached_file1.contents == u'some content in a file' )
    vassert( attached_file3.contents == u'even more content' )

@test(ExampleFixture.slots)
def slots(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open('/')

    expected_main_contents = u'In this slot will be some main content for the view on /'
    expected_secondary_contents = u'Some secondary content related to /'
    main_contents = fixture.get_main_slot_contents()
    vassert( main_contents == expected_main_contents )
    secondary_contents = fixture.get_secondary_slot_contents()
    vassert( secondary_contents == expected_secondary_contents )
    
    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'slots1.png'))

    fixture.driver_browser.click(XPath.link_with_text(u'Page 2'))
    expected_main_contents = u'This could, for example, be where a photo gallery shows a large photo.'
    expected_secondary_contents = u'Thumbnails will then sit on the side of the big photo.'
    main_contents = fixture.get_main_slot_contents()
    vassert( main_contents == expected_main_contents )
    secondary_contents = fixture.get_secondary_slot_contents()
    vassert( secondary_contents == expected_secondary_contents )

    fixture.driver_browser.capture_cropped_screenshot(fixture.new_screenshot_path(u'slots2.png'))

@test(ExampleFixture.basichtmlinputs)
def basichtmlinputs(fixture):
    fixture.start_example_app()
    fixture.driver_browser.open('/')
    
@test(Fixture)
def model_examples(fixture):
    # These examples are built to run outside of our infrastructure, hence have to be run like this:
    for example in ['modeltests1.py', 'modeltests2.py', 'modeltests3.py']:
        subprocess.check_call(['nosetests', 'reahl/doc/examples/tutorial/%s' % example ])

@test(ExampleFixture.addressbook1)
def test_addressbook1(fixture):
    browser = Browser(fixture.webapp)
    browser.open('/')
    
    vassert( browser.is_element_present(XPath.inputgroup_labelled(u'Add an address')) ) 

@test(ExampleFixture.addressbook2)
def test_addressbook2(fixture):
    browser = Browser(fixture.webapp)
    browser.open('/')
    
    browser.type(XPath.input_labelled(u'Name'), u'John') 
    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.click(XPath.button_labelled(u'Save'))
    
    vassert( browser.is_element_present(XPath.paragraph_containing(u'John: johndoe@some.org')) )

@test(ExampleFixture.pageflow1)
def test_pageflow1(fixture):
    browser = Browser(fixture.webapp)
    browser.open('/')

    vassert( browser.is_element_present(u'//ul[contains(@class,"reahl-menu")]') )

    browser.click(XPath.link_with_text(u'Add an address'))
    vassert( browser.location_path == u'/add' )
    
    browser.type(XPath.input_labelled(u'Name'), u'John') 
    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.click(XPath.button_labelled(u'Save'))
    
    vassert( browser.location_path == u'/add' )

    browser.click(XPath.link_with_text(u'Addresses'))
    vassert( browser.location_path == u'/' )
    vassert( browser.is_element_present(XPath.paragraph_containing(u'John: johndoe@some.org')) )


@test(ExampleFixture.pageflow2)
def test_pageflow2(fixture):
    browser = Browser(fixture.webapp)
    browser.open('/')

    vassert( browser.is_element_present(u'//ul[contains(@class,"reahl-menu")]') )

    browser.click(XPath.link_with_text(u'Add an address'))
    vassert( browser.location_path == u'/add' )
    
    browser.type(XPath.input_labelled(u'Name'), u'John') 
    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.click(XPath.button_labelled(u'Save'))
    
    vassert( browser.location_path == u'/' )
    vassert( browser.is_element_present(XPath.paragraph_containing(u'John: johndoe@some.org')) )


@test(ExampleFixture.parameterised1)
def test_parameterised1(fixture):
    browser = Browser(fixture.webapp)
    browser.open('/')

    browser.click(XPath.link_with_text(u'Add an address'))
    browser.type(XPath.input_labelled(u'Name'), u'John') 
    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.click(XPath.button_labelled(u'Save'))
    
    vassert( browser.location_path == u'/' )
    browser.click(XPath.link_with_text(u'edit'))

    john = parameterised1.Address.query.one()
    vassert( browser.location_path == u'/edit/%s' % john.id )
    browser.type(XPath.input_labelled(u'Name'), u'Johnny') 
    browser.type(XPath.input_labelled(u'Email'), u'johnny@walker.org')
    browser.click(XPath.button_labelled(u'Update'))
    
    vassert( browser.location_path == u'/' )
    vassert( browser.is_element_present(XPath.paragraph_containing(u'Johnny: johnny@walker.org')) )

@test(ExampleFixture.parameterised2)
def test_parameterised2(fixture):
    browser = Browser(fixture.webapp)
    browser.open('/')

    browser.click(XPath.link_with_text(u'Add an address'))
    browser.type(XPath.input_labelled(u'Name'), u'John') 
    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    browser.click(XPath.button_labelled(u'Save'))
    
    vassert( browser.location_path == u'/' )
    browser.click(XPath.button_labelled(u'Edit'))

    john = parameterised2.Address.query.one()
    vassert( browser.location_path == u'/edit/%s' % john.id )
    browser.type(XPath.input_labelled(u'Name'), u'Johnny') 
    browser.type(XPath.input_labelled(u'Email'), u'johnny@walker.org')
    browser.click(XPath.button_labelled(u'Update'))
    
    vassert( browser.location_path == u'/' )
    vassert( browser.is_element_present(XPath.paragraph_containing(u'Johnny: johnny@walker.org')) )








from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, P, H, Panel
from reahl.web.pager import SequentialPageIndex, PageMenu, PagedPanel


class AddressBookApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')
        find = self.define_view(u'/', title=u'Addresses')
        find.set_slot(u'main', AddressBookPanel.factory())


class AddressBookPanel(Panel):
    def __init__(self, view):
        super(AddressBookPanel, self).__init__(view)

        self.add_child(H(view, 1, text=u'Addresses'))

        self.page_index = SequentialPageIndex(Address.all_addresses(), items_per_page=5)
        self.address_list = AddressList(view, self.page_index)
        self.page_menu = PageMenu(view, u'page_menu', self.page_index, self.address_list)
        self.add_children([self.page_menu, self.address_list])


class AddressList(PagedPanel):
    def __init__(self, view, page_index):
        super(AddressList, self).__init__(view, page_index, 'addresslist')
        for address in self.current_contents:
            self.add_child(P(view, text=u'%s: %s' % (address.name, address.email_address)))


class Address(object):
    def __init__(self, name, email_address):
        self.name = name
        self.email_address = email_address
        
    @classmethod
    def all_addresses(cls):
        return [Address('friend %s' % i,'friend%s@some.org' % i ) for i in range(200)]





import elixir

from reahl.sqlalchemysupport import Session, metadata

from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, P, H, InputGroup, HMenu
from reahl.component.modelinterface import exposed, EmailField, Field, Event, Action
from reahl.systemaccountmodel import AccountManagementInterface


class AddressBookPage(TwoColumnPage):
    def __init__(self, view, main_bookmarks):
        super(AddressBookPage, self).__init__(view, style=u'basic')
        self.header.add_child(HMenu.from_bookmarks(view, main_bookmarks))


class AddressBookApp(Region):
    def assemble(self):
        addresses = self.define_view(u'/', title=u'Addresses')
        addresses.set_slot(u'main', AddressBookPanel.factory())

        add = self.define_view(u'/add', title=u'Add an address')
        add.set_slot(u'main', AddAddressForm.factory())

        self.define_transition(Address.events.save, add, addresses)

        bookmarks = [f.as_bookmark(self) for f in [addresses, add]]
        self.define_main_window(AddressBookPage, bookmarks)


class AddressBookPanel(Panel):
    def __init__(self, view):
        super(AddressBookPanel, self).__init__(view)

        self.add_child(H(view, 1, text=u'Addresses'))
        
        for address in Address.query.all():
            self.add_child(AddressBox(view, address))


class AddAddressForm(Form):
    def __init__(self, view):
        super(AddAddressForm, self).__init__(view, u'add_form')

        new_address = Address()

        grouped_inputs = self.add_child(InputGroup(view, label_text=u'Add an address'))
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_address.fields.name)) )
        grouped_inputs.add_child( LabelledBlockInput(TextInput(self, new_address.fields.email_address)) )

        grouped_inputs.add_child(Button(self, new_address.events.save))


class AddressBox(Panel):
    def __init__(self, view, address):
        super(AddressBox, self).__init__(view)
        self.add_child(P(view, text=u'%s: %s' % (address.name, address.email_address)))


class Address(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    elixir.using_mapper_options(save_on_init=False)
    
    email_address = elixir.Field(elixir.UnicodeText)
    name          = elixir.Field(elixir.UnicodeText)

    @exposed
    def fields(self, fields):
        fields.name = Field(label=u'Name', required=True)
        fields.email_address = EmailField(label=u'Email', required=True)

    def save(self):
        Session.add(self)
        
    @exposed(u'save')
    def events(self, events):
        events.save = Event(label=u'Save', action=Action(self.save))








# To run this test do:
# nosetests -F reahl.webdev.fixtures:BrowserSetup -s --nologcapture reahl/doc_dev/tutorialtests/ajaxtests.py
#



from reahl.tofu import test
from reahl.web_dev.fixtures import WebFixture
from reahl.webdev.tools import XPath
from reahl.doc.examples.tutorial.ajax.ajax import WidgetRefreshApp


class RefreshFixture(WebFixture):
    def new_browser(self):
        return self.driver_browser
        
    def new_webapp(self):
        return super(RefreshFixture, self).new_webapp(site_root=WidgetRefreshApp, enable_js=True)

    def text_shows_selected(self, expected_selected):
        return self.browser.is_element_present(XPath.paragraph_containing(u'You selected link number %s' % expected_selected))

#------ Tests

@test(RefreshFixture)
def refreshing_widget(fixture):
    """Clicking on a link, refreshes the displayed text to indicate which link 
       was clicked, without triggering a page load."""

    fixture.reahl_server.set_app(fixture.webapp)
    browser = fixture.browser

    browser.open(u'/')

    assert fixture.text_shows_selected(1)    
    assert not fixture.text_shows_selected(3)

    with browser.no_page_load_expected():
        browser.click(XPath.link_with_text(u'Select 3'))

    assert not fixture.text_shows_selected(1)    
    assert fixture.text_shows_selected(3)    







# Copyright 2012, 2013 Reahl Software Services (Pty) Ltd. All rights reserved.
#
#    This file is part of Reahl.
#
#    Reahl is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation; version 3 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import elixir

from reahl.sqlalchemysupport import Session, metadata

from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Form, TextInput, LabelledBlockInput, Button, Panel, \
                         P, H, InputGroup, FileUploadInput, SimpleFileInput
from reahl.component.modelinterface import exposed, EmailField, Field, Event, Action, FileField


class FileUploadApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')
        home = self.define_view(u'/', title=u'File upload demo')
        home.set_slot(u'main', CommentPostPanel.factory())


class Comment(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    elixir.using_mapper_options(save_on_init=False)
    
    email_address  = elixir.Field(elixir.UnicodeText)
    text           = elixir.Field(elixir.UnicodeText)
    attached_files = elixir.OneToMany(u'AttachedFile')
    
    @exposed
    def fields(self, fields):
        fields.email_address = EmailField(label=u'Email address', required=True)
        fields.text          = Field(label=u'Comment', required=True)
        fields.uploaded_files = FileField(allow_multiple=True, max_size_bytes=4*1000*1000, max_files=4, accept=[u'text/*'])

    @exposed
    def events(self, events):
        events.submit = Event(label=u'Submit', action=Action(self.submit))

    def attach_uploaded_files(self):
        for f in self.uploaded_files:
            self.attached_files.append(AttachedFile(comment=self, filename=f.filename, contents=f.file_obj.read()))

    def submit(self):
        self.attach_uploaded_files()
        Session.add(self)


class AttachedFile(elixir.Entity):
    elixir.using_options(session=Session, metadata=metadata)
    
    filename = elixir.Field(elixir.UnicodeText)
    contents = elixir.Field(elixir.LargeBinary)
    comment  = elixir.ManyToOne(Comment)

class CommentPostPanel(Panel):
    def __init__(self, view):
        super(CommentPostPanel, self).__init__(view)

        self.add_child(CommentForm(view))

        for comment in Comment.query.all():
            self.add_child(CommentBox(view, comment))


class CommentForm(Form):
    def __init__(self, view):
        super(CommentForm, self).__init__(view, u'myform')

        new_comment = Comment()
        self.add_child(H(view, 1, text=u'Leave a comment'))
        self.add_child(LabelledBlockInput(TextInput(self, new_comment.fields.email_address)))
        self.add_child(LabelledBlockInput(TextInput(self, new_comment.fields.text)))

        attachments = self.add_child(InputGroup(view, label_text=u'Attach files'))
        attachments.add_child(FileUploadInput(self, new_comment.fields.uploaded_files))

        self.define_event_handler(new_comment.events.submit)
        buttons = self.add_child(InputGroup(view, label_text=u' '))
        buttons.add_child(Button(self, new_comment.events.submit))


class CommentBox(Panel):
    def __init__(self, view, comment):
        super(CommentBox, self).__init__(view)
        self.add_child(P(view, text=u'By %s: %s' % (comment.email_address, comment.text)))
        





# To run this test do:
# nosetests -F reahl.webdev.fixtures:BrowserSetup -s --nologcapture reahl/doc_dev/tutorialtests/accesstests1.py


from reahl.tofu import test
from reahl.web_dev.fixtures import WebFixture
from reahl.systemaccountmodel import EmailAndPasswordSystemAccount

from reahl.doc.examples.tutorial.access1.access1 import AddressBook, Address


class AccessFixture(WebFixture):
    password = u'bobbejaan'

    def new_account(self, email=u'johndoe@some.org'):
        account = EmailAndPasswordSystemAccount(email=email)
        account.set_new_password(account.email, self.password)
        account.activate()
        return account

    def new_address_book(self, owner=None):
        owner = owner or self.account
        return AddressBook(owner=owner)

    def new_other_account(self):
        return self.new_account(email=u'other@some.org')
        
    def new_other_address_book(self):
        return self.new_address_book(owner=self.other_account)



@test(AccessFixture)
def separate_address_books(fixture):
    """An Address is created in a particular AddressBook, which is owned by a SystemAccount."""
    
    account = fixture.account
    address_book = fixture.address_book
    other_address_book = fixture.other_address_book

    # AddressBooks are owned
    address_book.owner is account
    other_address_book.owner is fixture.other_account

    # Addresses live in specific AddressBooks
    assert address_book.addresses == []
    assert other_address_book.addresses == []

    address1 = Address(address_book=address_book, email_address=u'friend1@some.org', name=u'Friend1')
    address2 = Address(address_book=address_book, email_address=u'friend2@some.org', name=u'Friend2')

    address3 = Address(address_book=other_address_book, email_address=u'friend3@some.org', name=u'Friend3')

    for address in [address1, address2, address3]:
        address.save()

    assert address_book.addresses == [address1, address2]
    assert other_address_book.addresses == [address3]


@test(AccessFixture)
def collaborators(fixture):
    """A particular SystemAccount can see its own AddressBooks as well as all the AddressBooks
       it is explicitly allowed to see, but no other AddressBooks."""
    
    account = fixture.account
    address_book = fixture.address_book
    other_address_book = fixture.other_address_book

    unrelated_account = fixture.new_account(email=u'unrelated@some.org')
    unrelated_address_book = fixture.new_address_book(owner=unrelated_account)
    
    other_address_book.allow(account)
    
    # Checks to see whether an AddressBook is visible
    assert address_book.is_visible_to(account)
    assert other_address_book.is_visible_to(account)
    assert not unrelated_address_book.is_visible_to(account)

    # Getting a list of visible AddressBooks (for populating the screen)
    books = AddressBook.address_books_visible_to(account)
    assert set(books) == set([address_book, other_address_book])


@test(AccessFixture)
def collaborator_rights(fixture):
    """When allowing an account to see another's AddressBook, the rights it has to the AddressBook are specified."""

    account = fixture.account
    address_book = fixture.address_book
    other_address_book = fixture.other_address_book

    # Case: defaults
    other_address_book.allow(account)
    assert not other_address_book.can_be_edited_by(account)
    assert not other_address_book.can_be_added_to_by(account)

    # Case: rights specified
    other_address_book.allow(account, can_edit_addresses=True, can_add_addresses=True)
    assert other_address_book.can_be_edited_by(account)
    assert other_address_book.can_be_added_to_by(account)


@test(AccessFixture)
def adding_collaborators(fixture):
    """The owner of an AddressBook may add collaborators to it."""

    account = fixture.account
    address_book = fixture.address_book
    other_address_book = fixture.other_address_book

    assert address_book.collaborators_can_be_added_by(account)
    assert not other_address_book.collaborators_can_be_added_by(account)






from nose.tools import istest

class Address(object):
    def __init__(self, name, email_address):
        self.name = name
        self.email_address = email_address

    def save(self, address_book):
        address_book.add_address(self)


class AddressBook(object):
    def __init__(self):
        self.addresses = []

    def add_address(self, address):
        self.addresses.append(address)




@istest
def test_model():
    contacts = AddressBook()
    Address(u'John', u'john@world.com').save(contacts)
    Address(u'Jane', u'jane@world.com').save(contacts)

    assert contacts.addresses[0].name == u'John'
    assert contacts.addresses[0].email_address == u'john@world.com'

    #import pdb; pdb.set_trace()
    assert contacts.addresses[1].name == u'Jane'
    assert contacts.addresses[1].email_address == u'jane@world.com'








from reahl.tofu import test

from reahl.web_dev.fixtures import WebFixture
from reahl.webdev.tools import Browser, XPath

from reahl.doc.examples.tutorial.parameterised2.parameterised2 import AddressBookApp, Address


@test(WebFixture)
def adding_an_address(fixture):
    """To add a new address, a user clicks on "Add Address" link on the menu, then supplies the 
       information for the new address and clicks the Save button. Upon successful addition of the
       address, the user is returned to the home page where the new address is now listed."""

    browser = Browser(fixture.new_webapp(site_root=AddressBookApp))

    browser.open(u'/')
    browser.click(XPath.link_with_text(u'Add an address'))

    actual_title = browser.title
    assert actual_title == u'Add an address', u'Expected to be on the Add an address page'

    browser.type(XPath.input_labelled(u'Name'), u'John Doe')
    browser.type(XPath.input_labelled(u'Email'), u'johndoe@some.org')
    
    browser.click(XPath.button_labelled(u'Save'))
    
    actual_title = browser.title
    assert actual_title == u'Addresses', u'Expected to be back on the home page after editing'
            
    assert browser.is_element_present(XPath.paragraph_containing(u'John Doe: johndoe@some.org')), \
           u'Expected the newly added address to be listed on the home page'




# Copyright 2012, 2013 Reahl Software Services (Pty) Ltd. All rights reserved.
#
#    This file is part of Reahl.
#
#    Reahl is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation; version 3 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


from reahl.web.fw import Region
from reahl.web.ui import TwoColumnPage, Panel, P


class BasicHTMLWidgetsApp(Region):
    def assemble(self):
        self.define_main_window(TwoColumnPage, style=u'basic')  

        home = self.define_view(u'/', title=u'Basic HTML Widgets demo')
        home.set_slot(u'main', WidgetPanel.factory())


class WidgetPanel(Panel):
    def __init__(self, view):
        super(WidgetPanel, self).__init__(view)
        self.add_child(P(view, text=u'A paragraph'))

